@@ -17,13 +17,19 @@
 package org.eclipse.jst.j2ee.internal;
 
 import java.util.ArrayList;
-import java.util.Collections;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 
 import org.eclipse.core.commands.ExecutionException;
+import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IWorkspaceRoot;
+import org.eclipse.core.resources.IWorkspaceRunnable;
 import org.eclipse.core.resources.ResourcesPlugin;
+import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
@@ -39,8 +45,15 @@ import org.eclipse.jface.viewers.ICheckStateListener;
 import org.eclipse.jface.viewers.TableLayout;
 import org.eclipse.jst.j2ee.application.internal.operations.AddComponentToEnterpriseApplicationDataModelProvider;
 import org.eclipse.jst.j2ee.application.internal.operations.RemoveComponentFromEnterpriseApplicationDataModelProvider;
+import org.eclipse.jst.j2ee.application.internal.operations.UpdateManifestDataModelProperties;
+import org.eclipse.jst.j2ee.application.internal.operations.UpdateManifestDataModelProvider;
+import org.eclipse.jst.j2ee.commonarchivecore.internal.helpers.ArchiveManifest;
 import org.eclipse.jst.j2ee.internal.common.J2EEVersionUtil;
+import org.eclipse.jst.j2ee.internal.common.classpath.J2EEComponentClasspathUpdater;
+import org.eclipse.jst.j2ee.internal.plugin.IJ2EEModuleConstants;
 import org.eclipse.jst.j2ee.internal.plugin.J2EEUIMessages;
+import org.eclipse.jst.j2ee.internal.plugin.J2EEUIPlugin;
+import org.eclipse.jst.j2ee.internal.project.J2EEProjectUtilities;
 import org.eclipse.jst.j2ee.project.facet.IJavaProjectMigrationDataModelProperties;
 import org.eclipse.jst.j2ee.project.facet.JavaProjectMigrationDataModelProvider;
 import org.eclipse.swt.SWT;
@@ -58,7 +71,9 @@ import org.eclipse.swt.widgets.TableItem;
 import org.eclipse.swt.widgets.Text;
 import org.eclipse.wst.common.componentcore.ComponentCore;
 import org.eclipse.wst.common.componentcore.datamodel.properties.ICreateReferenceComponentsDataModelProperties;
+import org.eclipse.wst.common.componentcore.internal.builder.DependencyGraphManager;
 import org.eclipse.wst.common.componentcore.internal.operation.CreateReferenceComponentsDataModelProvider;
+import org.eclipse.wst.common.componentcore.internal.operation.RemoveReferenceComponentsDataModelProvider;
 import org.eclipse.wst.common.componentcore.internal.resources.VirtualArchiveComponent;
 import org.eclipse.wst.common.componentcore.resources.IVirtualComponent;
 import org.eclipse.wst.common.componentcore.resources.IVirtualReference;
@@ -167,41 +182,60 @@ public class AddModulestoEARPropertiesPage implements IJ2EEDependenciesControl,
 		IStatus stat = OK_STATUS;
 		try {
 			if( earComponent != null ){
-				List list = newJ2EEModulesToAdd();				
-				if (list != null && !list.isEmpty()) {
-					IDataModel dm = DataModelFactory.createDataModel(new AddComponentToEnterpriseApplicationDataModelProvider());
-					
-					dm.setProperty(ICreateReferenceComponentsDataModelProperties.SOURCE_COMPONENT, earComponent);					
-					dm.setProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST, list);
-					stat = dm.validateProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST);
-					if (stat != OK_STATUS)
-						return stat;
-					dm.getDefaultOperation().execute(monitor, null);
-				}
-				
-				if (!javaProjectsList.isEmpty()) {
-	
-					for (int i = 0; i < javaProjectsList.size(); i++) {
-						IProject proj = (IProject) javaProjectsList.get(i);
-						IDataModel migrationdm = DataModelFactory.createDataModel(new JavaProjectMigrationDataModelProvider());
-						migrationdm.setProperty(IJavaProjectMigrationDataModelProperties.PROJECT_NAME, proj.getName());
-						migrationdm.getDefaultOperation().execute(monitor, null);
-	
-	
-						IDataModel refdm = DataModelFactory.createDataModel(new CreateReferenceComponentsDataModelProvider());
-						List targetCompList = (List) refdm.getProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST);
-	
-						IVirtualComponent targetcomponent = ComponentCore.createComponent(proj);
-						targetCompList.add(targetcomponent);
-	
-						refdm.setProperty(ICreateReferenceComponentsDataModelProperties.SOURCE_COMPONENT, earComponent);
-						refdm.setProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST, targetCompList);
-						refdm.getDefaultOperation().execute(monitor, null);
-						j2eeComponentList.add(targetcomponent);
-					}
+				final List list = newJ2EEModulesToAdd();				
+				boolean shouldRun = (list != null && !list.isEmpty()) || !javaProjectsList.isEmpty();
+				if(shouldRun){
+					IWorkspaceRunnable runnable = new IWorkspaceRunnable(){
+
+						public void run(IProgressMonitor monitor) throws CoreException{
+							if (list != null && !list.isEmpty()) {
+								IDataModel dm = DataModelFactory.createDataModel(new AddComponentToEnterpriseApplicationDataModelProvider());
+								
+								dm.setProperty(ICreateReferenceComponentsDataModelProperties.SOURCE_COMPONENT, earComponent);					
+								dm.setProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST, list);
+								IStatus stat = dm.validateProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST);
+								if (stat != OK_STATUS)
+									throw new CoreException(stat);
+								try {
+									dm.getDefaultOperation().execute(monitor, null);
+								} catch (ExecutionException e) {
+									Logger.getLogger().log(e);
+								}
+							}
+							if (!javaProjectsList.isEmpty()) {
+								for (int i = 0; i < javaProjectsList.size(); i++) {
+									try {
+										IProject proj = (IProject) javaProjectsList.get(i);
+										IDataModel migrationdm = DataModelFactory.createDataModel(new JavaProjectMigrationDataModelProvider());
+										migrationdm.setProperty(IJavaProjectMigrationDataModelProperties.PROJECT_NAME, proj.getName());
+										migrationdm.getDefaultOperation().execute(monitor, null);
+
+
+										IDataModel refdm = DataModelFactory.createDataModel(new CreateReferenceComponentsDataModelProvider());
+										List targetCompList = (List) refdm.getProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST);
+
+										IVirtualComponent targetcomponent = ComponentCore.createComponent(proj);
+										targetCompList.add(targetcomponent);
+
+										refdm.setProperty(ICreateReferenceComponentsDataModelProperties.SOURCE_COMPONENT, earComponent);
+										refdm.setProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST, targetCompList);
+										
+										// referenced java projects should have archiveName attribute
+										((Map)refdm.getProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENTS_TO_URI_MAP)).put(targetcomponent, proj.getName() + IJ2EEModuleConstants.JAR_EXT);
+
+										refdm.getDefaultOperation().execute(monitor, null);
+										j2eeComponentList.add(targetcomponent);
+									} catch (ExecutionException e) {
+										Logger.getLogger().log(e);
+									}
+								}
+							}
+
+						}
+					};
+					J2EEUIPlugin.getWorkspace().run(runnable, monitor);
 				}
 			}
-
 		} catch (Exception e) {
 			Logger.getLogger().log(e);
 		}
@@ -214,8 +248,14 @@ public class AddModulestoEARPropertiesPage implements IJ2EEDependenciesControl,
 			List list = getComponentsToRemove();
 			if( !list.isEmpty()){
 				try {
+					// retrieve all dependencies on these components within the scope of the EAR
+					Map dependentComps = getEARModuleDependencies(earComponent, list);
+					// remove the components from the EAR
 					IDataModelOperation op = removeComponentFromEAROperation(earComponent, list);
 					op.execute(null, null);
+					// if that succeeded, remove all EAR-scope J2EE dependencies on these components
+					J2EEComponentClasspathUpdater.getInstance().queueUpdateEAR(earComponent.getProject());
+					removeEARComponentDependencies(dependentComps);
 				} catch (ExecutionException e) {
 					Logger.getLogger().log(e);
 				}
@@ -224,6 +264,91 @@ public class AddModulestoEARPropertiesPage implements IJ2EEDependenciesControl,
 		return stat;
 	}		
 	
+	private Map getEARModuleDependencies(final IVirtualComponent earComponent, final List components) {
+		final Map dependentComps = new HashMap();
+		// get all current references to project within the scope of this EAR
+		for (int i = 0; i < components.size(); i++) {
+			
+			final List compsForProject = new ArrayList();
+			final IVirtualComponent comp = (IVirtualComponent) components.get(i);
+			final IProject[] dependentProjects = DependencyGraphManager.getInstance().getDependencyGraph().getReferencingComponents(comp.getProject());
+			for (int j = 0; j < dependentProjects.length; j++) {
+				final IProject project = dependentProjects[j];
+				// if this is an EAR, can skip
+				if (J2EEProjectUtilities.isEARProject(project)) {
+					continue;
+				}
+				final IVirtualComponent dependentComp = ComponentCore.createComponent(project);
+				// ensure that the project's share an EAR
+				final IProject[] refEARs = J2EEProjectUtilities.getReferencingEARProjects(project);
+				boolean sameEAR = false;
+				for (int k = 0; k < refEARs.length; k++) {
+					if (refEARs[k].equals(earComponent.getProject())) {
+						sameEAR = true;
+						break;
+					}
+				}
+				if (!sameEAR) {
+					continue;
+				}
+				// if the dependency is a web lib dependency, can skip
+				if (J2EEProjectUtilities.isDynamicWebProject(project)) {
+					IVirtualReference ref = dependentComp.getReference(comp.getName());
+					if (ref != null && ref.getRuntimePath().equals(new Path("/WEB-INF/lib"))) { //$NON-NLS-1$
+						continue;
+					}
+				}
+				compsForProject.add(dependentComp);
+			}
+			dependentComps.put(comp, compsForProject);
+		}
+		return dependentComps;
+	}
+	
+	private void removeEARComponentDependencies(final Map dependentComps) throws ExecutionException {
+		final Iterator targets = dependentComps.keySet().iterator();
+		while (targets.hasNext()) {
+			final IVirtualComponent target = (IVirtualComponent) targets.next();
+			final List sources = (List) dependentComps.get(target);
+			for (int i = 0; i < sources.size(); i++) {
+				final IVirtualComponent source = (IVirtualComponent) sources.get(i);
+				final IDataModel model = DataModelFactory.createDataModel(new RemoveReferenceComponentsDataModelProvider());
+				model.setProperty(ICreateReferenceComponentsDataModelProperties.SOURCE_COMPONENT, source);
+				final List modHandlesList = (List) model.getProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST);
+				modHandlesList.add(target);
+				model.setProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST, modHandlesList);
+				model.getDefaultOperation().execute(null, null);
+				// update the manifest
+				removeManifestDependency(source, target);
+			}
+		}
+	}
+	
+	private void removeManifestDependency(final IVirtualComponent source, final IVirtualComponent target) 
+		throws ExecutionException {
+		final String sourceProjName = source.getProject().getName();
+		final String targetProjName = target.getProject().getName();
+		final IProgressMonitor monitor = new NullProgressMonitor();
+		final IFile manifestmf = J2EEProjectUtilities.getManifestFile(source.getProject());
+		final ArchiveManifest mf = J2EEProjectUtilities.readManifest(source.getProject());
+		if (mf == null)
+			return;
+		final IDataModel updateManifestDataModel = DataModelFactory.createDataModel(new UpdateManifestDataModelProvider());
+		updateManifestDataModel.setProperty(UpdateManifestDataModelProperties.PROJECT_NAME, sourceProjName);
+		updateManifestDataModel.setBooleanProperty(UpdateManifestDataModelProperties.MERGE, false);
+		updateManifestDataModel.setProperty(UpdateManifestDataModelProperties.MANIFEST_FILE, manifestmf);
+		String[] cp = mf.getClassPathTokenized();
+		List cpList = new ArrayList();
+		String cpToRemove = targetProjName + ".jar";//$NON-NLS-1$
+		for (int i = 0; i < cp.length; i++) {
+			if (!cp[i].equals(cpToRemove)) {
+				cpList.add(cp[i]);
+			}
+		}
+		updateManifestDataModel.setProperty(UpdateManifestDataModelProperties.JAR_LIST, cpList);
+		updateManifestDataModel.getDefaultOperation().execute(monitor, null );
+	}
+	
 	protected IDataModelOperation removeComponentFromEAROperation(IVirtualComponent sourceComponent, List targetComponentsHandles) {
 		IDataModel model = DataModelFactory.createDataModel(new RemoveComponentFromEnterpriseApplicationDataModelProvider());
 		model.setProperty(ICreateReferenceComponentsDataModelProperties.SOURCE_COMPONENT, sourceComponent);
@@ -462,7 +587,7 @@ public class AddModulestoEARPropertiesPage implements IJ2EEDependenciesControl,
 		Object[] elements = availableComponentsViewer.getCheckedElements();
 		List list;
 		if (elements == null || elements.length == 0)
-			list = Collections.EMPTY_LIST;
+			list = new ArrayList(0); // Collections.EMPTY_LIST would cause UnsupportedOperationException when a later attempt to add to it is made
 		else {
 			list = new ArrayList();
 			for (int i = 0; i < elements.length; i++) {
@@ -478,7 +603,7 @@ public class AddModulestoEARPropertiesPage implements IJ2EEDependenciesControl,
 		Object[] elements = availableComponentsViewer.getCheckedElements();
 		List list;
 		if (elements == null || elements.length == 0)
-			list = Collections.EMPTY_LIST;
+			list = new ArrayList(0); // Collections.EMPTY_LIST would cause UnsupportedOperationException when a later attempt to add to it is made
 		else {
 			list = new ArrayList();
 			for (int i = 0; i < elements.length; i++) {
@@ -528,9 +653,14 @@ public class AddModulestoEARPropertiesPage implements IJ2EEDependenciesControl,
 		availableComponentsViewer.getTable().setLayoutData(data);
 
 		TableItem [] items = availableComponentsViewer.getTable().getItems();
-
 		List list = new ArrayList();
-		Object[] comps = getComponentsInEar();
+		//Object[] comps = getComponentsInEar();
+		
+		if( j2eeComponentList.isEmpty() ){
+			Object[] comps = getComponentsInEar();
+			j2eeComponentList.addAll( Arrays.asList(comps));
+			}
+			Object[] comps = j2eeComponentList.toArray();
 		
 		for( int i=0; i< items.length; i++ ){
 			Object element = items[i].getData();
@@ -546,7 +676,7 @@ public class AddModulestoEARPropertiesPage implements IJ2EEDependenciesControl,
 		}
 		
 		availableComponentsViewer.setCheckedElements(list.toArray());
-		j2eeComponentList.addAll(list);
+	//	j2eeComponentList.addAll(list);
 		GridData btndata = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_BEGINNING);
 		buttonColumn.setLayoutData(btndata);
 
