@@ -23,6 +23,10 @@ import org.eclipse.core.resources.IContainer;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.Path;
+import org.eclipse.core.runtime.jobs.ISchedulingRule;
+import org.eclipse.core.runtime.jobs.MultiRule;
 import org.eclipse.emf.common.command.Command;
 import org.eclipse.emf.common.util.EList;
 import org.eclipse.emf.ecore.EObject;
@@ -51,21 +55,24 @@ import org.eclipse.jst.j2ee.componentcore.util.EARArtifactEdit;
 import org.eclipse.jst.j2ee.internal.J2EEConstants;
 import org.eclipse.jst.j2ee.internal.J2EEVersionConstants;
 import org.eclipse.jst.j2ee.internal.project.J2EEProjectUtilities;
+import org.eclipse.jst.j2ee.model.internal.validation.EARValidationMessageResourceHandler;
 import org.eclipse.jst.j2ee.model.internal.validation.EarValidator;
 import org.eclipse.jst.j2ee.webservice.wsclient.ServiceRef;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.wst.common.componentcore.ComponentCore;
 import org.eclipse.wst.common.componentcore.internal.impl.ModuleURIUtil;
+import org.eclipse.wst.common.componentcore.internal.util.IModuleConstants;
 import org.eclipse.wst.common.componentcore.resources.IVirtualComponent;
 import org.eclipse.wst.common.componentcore.resources.IVirtualFile;
 import org.eclipse.wst.common.componentcore.resources.IVirtualReference;
 import org.eclipse.wst.common.internal.emfworkbench.WorkbenchResourceHelper;
-import org.eclipse.wst.validation.internal.core.Message;
 import org.eclipse.wst.validation.internal.core.ValidationException;
 import org.eclipse.wst.validation.internal.operations.IWorkbenchContext;
+import org.eclipse.wst.validation.internal.operations.LocalizedMessage;
 import org.eclipse.wst.validation.internal.provisional.core.IMessage;
 import org.eclipse.wst.validation.internal.provisional.core.IReporter;
 import org.eclipse.wst.validation.internal.provisional.core.IValidationContext;
-import org.eclipse.wst.validation.internal.provisional.core.MessageLimitException;
+import org.eclipse.wst.validation.internal.provisional.core.IValidatorJob;
 
 
 /**
@@ -73,9 +80,12 @@ import org.eclipse.wst.validation.internal.provisional.core.MessageLimitExceptio
  * 
  * @author: Administrator
  */
-public class UIEarValidator extends EarValidator implements UIEarMessageConstants {
+public class UIEarValidator extends EarValidator {
 	public static final String VALIDATOR_ID = "org.eclipse.jst.j2ee.internal.validation.UIEarValidator"; //$NON-NLS-1$
 	public static final String MANIFEST_GROUP_NAME = "WSAD.EAR.MANIFEST"; //$NON-NLS-1$
+	public static final String DOCTYPE_1_2 = "1.2"; //$NON-NLS-1$
+	public static final String DOCTYPE_1_3 = "1.3"; //$NON-NLS-1$
+	
 	protected UIEarHelper earHelper;
 //	private EARArtifactEdit earEdit = null;
 	private IProject project = null;
@@ -104,7 +114,9 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 		params[0] = projectName;
 		params[1] = earProjectName;
 		params[2] = moduleUri;
-		addError(getBaseName(), DUPLICATE_MODULE_FOR_PROJECT_NAME_ERROR_, params, appDD);
+		String msg = NLS.bind(EARValidationMessageResourceHandler.DUPLICATE_MODULE_FOR_PROJECT_NAME_ERROR_, params);
+	
+		addLocalizedError(msg, appDD);
 	}
 
 	/**
@@ -164,7 +176,8 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 		String[] params = new String[2];
 		params[0] = anArchive.getURI();
 		params[1] = entry;
-		addWarning(getBaseName(), INVALID_MANIFEST_CLASSPATH_ONE_WARN_, params, getManifestFile(anArchive), MANIFEST_GROUP_NAME);
+		String msg = NLS.bind(EARValidationMessageResourceHandler.INVALID_MANIFEST_CLASSPATH_ONE_WARN_, params);
+		addLocalizedWarning(msg, getManifestFile(anArchive), MANIFEST_GROUP_NAME);
 	}
 
 	protected void invalidClassPathEntryWarning(String entry, String resolvedEntry, Archive anArchive) {
@@ -172,7 +185,8 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 		params[0] = anArchive.getURI();
 		params[1] = entry;
 		params[2] = resolvedEntry;
-		addWarning(getBaseName(), INVALID_MANIFEST_CLASSPATH_TWO_WARN_, params, getManifestFile(anArchive), MANIFEST_GROUP_NAME);
+		String msg = NLS.bind(EARValidationMessageResourceHandler.INVALID_MANIFEST_CLASSPATH_TWO_WARN_, params);		
+		addLocalizedWarning(msg, getManifestFile(anArchive), MANIFEST_GROUP_NAME);
 	}
 
 	protected void invalidDepedencyWarning(String entry, Archive anArchive, ModuleFile m) {
@@ -180,7 +194,9 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 		params[0] = m.getURI();
 		params[1] = entry;
 		params[2] = anArchive.getURI();
-		addWarning(getBaseName(), INVALID_MANIFEST_CLASSPATH_DEPENDENCY_WARN_, params, getManifestFile(anArchive), MANIFEST_GROUP_NAME);
+		String msg = NLS.bind(EARValidationMessageResourceHandler.INVALID_MANIFEST_CLASSPATH_DEPENDENCY_WARN_, params);
+		
+		addLocalizedWarning(msg, getManifestFile(anArchive), MANIFEST_GROUP_NAME);
 	}
 
 	/**
@@ -193,14 +209,15 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 		earHelper = newEarHelper;
 	}
 
-	public void validate(IValidationContext inHelper, IReporter inReporter) throws org.eclipse.wst.validation.internal.core.ValidationException {
-		earHelper = ((UIEarHelper) inHelper);
+	public IStatus validateInJob(IValidationContext inHelper, IReporter inReporter) throws org.eclipse.wst.validation.internal.core.ValidationException {
+
 		IProject proj = ((IWorkbenchContext) inHelper).getProject();
 		IVirtualComponent earModule = ComponentCore.createComponent(proj);
             if(J2EEProjectUtilities.isEARProject(proj)){
 				IVirtualFile ddFile = earModule.getRootFolder().getFile(J2EEConstants.APPLICATION_DD_URI);
-				if( ddFile.exists()) {				
-					super.validate(inHelper, inReporter);
+				if( ddFile.exists()) {	
+					IStatus status = IValidatorJob.OK_STATUS;
+					status = super.validateInJob(inHelper, inReporter);
 					validateModuleMaps(earModule);
 					validateManifests();
 	//				validateUtilJarMaps(earEdit,earModule);
@@ -208,29 +225,61 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 	//				validateDocType(earEdit,earModule);					
 				}
             }
-		
+		return IValidatorJob.OK_STATUS;
 	}	
 
-
+	public ISchedulingRule getSchedulingRule(IValidationContext helper) {
+		
+		ISchedulingRule combinedRule = null;
+		IProject project = ((IWorkbenchContext) helper).getProject();
+		IVirtualComponent comp = ComponentCore.createComponent( project );
+		IFile appDeploymentDescriptor = null;
+		if( comp != null ){
+			IVirtualFile vf = comp.getRootFolder().getFile(new Path(J2EEConstants.APPLICATION_DD_URI));
+			if( vf!= null ){
+				appDeploymentDescriptor = vf.getUnderlyingFile();
+				combinedRule = MultiRule.combine(appDeploymentDescriptor, combinedRule);
+			}
+			IVirtualReference[] refs = comp.getReferences();
+			for( int i=0; i< refs.length; i++ ){
+				IVirtualComponent refComp = refs[i].getReferencedComponent();
+				if( refComp != null && !refComp.isBinary()){
+					String type = J2EEProjectUtilities.getJ2EEComponentType( refComp );
+					IVirtualFile refDDFile = null;
+					if( type.equals(IModuleConstants.JST_WEB_MODULE)){
+						refDDFile = refComp.getRootFolder().getFile(new Path(J2EEConstants.WEBAPP_DD_URI));
+					}else if ( type.equals(IModuleConstants.JST_CONNECTOR_MODULE)){
+						refDDFile = refComp.getRootFolder().getFile(new Path(J2EEConstants.RAR_DD_URI));
+					}else if( type.equals(IModuleConstants.JST_EJB_MODULE)){
+						refDDFile = refComp.getRootFolder().getFile(new Path(J2EEConstants.EJBJAR_DD_URI));
+					}else if( type.equals(IModuleConstants.JST_APPCLIENT_MODULE)){
+						refDDFile = refComp.getRootFolder().getFile(new Path(J2EEConstants.APPLICATION_DD_URI));
+					}
+					if( refDDFile!= null ){
+						IFile dd = refDDFile.getUnderlyingFile();
+						combinedRule = MultiRule.combine(dd, combinedRule);
+					}						
+				}
+			}
+			
+		}
+		return combinedRule;
+	}
 
 	public void validateManifests() throws ValidationException {
-		
 		if (earFile == null)
 			return;
 		List archives = earFile.getArchiveFiles();
 		for (int i = 0; i < archives.size(); i++) {
-			try {
-				Archive anArchive = (Archive) archives.get(i);
 
-					IFile target = getManifestFile(anArchive);
-					if (target != null)
-						_reporter.removeMessageSubset(this, target, MANIFEST_GROUP_NAME);
-					validateManifestCase(anArchive);
-					validateManifestLines(anArchive);
-					validateManifestClasspath(anArchive);
+			Archive anArchive = (Archive) archives.get(i);
 
-			} catch (MessageLimitException me) {
-			}
+			IFile target = getManifestFile(anArchive);
+			if (target != null)
+				_reporter.removeMessageSubset(this, target, MANIFEST_GROUP_NAME);
+			validateManifestCase(anArchive);
+			validateManifestLines(anArchive);
+			validateManifestClasspath(anArchive);
 		}
 	}
 
@@ -253,7 +302,8 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 			if (mfuri.equalsIgnoreCase(uri) && !mfuri.equals(uri)) {
 				String[] params = {uri, anArchive.getURI()};
 				IResource target = earHelper.getProject().getFile(J2EEConstants.MANIFEST_URI);
-				addError(getBaseName(), INVALID_CASE_FOR_MANIFEST_ERROR_, params, target);
+				String msg = NLS.bind(EARValidationMessageResourceHandler.INVALID_CASE_FOR_MANIFEST_ERROR_, params);
+				addLocalizedError(msg, target);				
 			}
 		}
 
@@ -268,7 +318,9 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 			//mf.printStackTrace();
 			mf.getMessage();
 			String[] args = new String[]{anArchive.getURI()};
-			addError(ERROR_READING_MANIFEST_ERROR_, args);
+			String tmp = NLS.bind(EARValidationMessageResourceHandler.ERROR_READING_MANIFEST_ERROR_, args);
+			
+			addLocalizedError(tmp, args);
 		}
 		
 		if(manifest == null)
@@ -347,23 +399,32 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 		String[] args = new String[2];
 		args[0] = Integer.toString(lineNo);
 		args[1] = anArchive.getURI();
-		if (target != null)
-			addError(getBaseName(), MANIFEST_LINE_EXCEEDS_LENGTH_ERROR_, args, target, MANIFEST_GROUP_NAME, lineNo);
-		else
-			addError(getBaseName(), MANIFEST_LINE_EXCEEDS_LENGTH_ERROR_, args, null, MANIFEST_GROUP_NAME);
+		
+		String tmp = NLS.bind(EARValidationMessageResourceHandler.MANIFEST_LINE_EXCEEDS_LENGTH_ERROR_, args);
+		
+		if( lineNo >= 0 ){
+			addLocalizedError(tmp, target, MANIFEST_GROUP_NAME, lineNo );
+		}else{
+			addLocalizedError(tmp, target, MANIFEST_GROUP_NAME );
+		}
+
 	}
 
 	protected void addFileEndError(Archive anArchive, ManifestLineValidator mfVal, IFile target) {
 		String[] args = new String[]{anArchive.getURI()};
+		
+		String tmp = NLS.bind(EARValidationMessageResourceHandler.MANIFEST_LINE_END_ERROR_, args);
+		
 		if (target != null)
-			addError(getBaseName(), MANIFEST_LINE_END_ERROR_, args, getManifestFile(anArchive), MANIFEST_GROUP_NAME, mfVal.getLineCount());
+			addLocalizedError(tmp, getManifestFile(anArchive), MANIFEST_GROUP_NAME, mfVal.getLineCount());
 		else
-			addError(getBaseName(), MANIFEST_LINE_END_ERROR_, args, MANIFEST_GROUP_NAME);
+			addLocalizedError(tmp, null, MANIFEST_GROUP_NAME);
 	}
 
 	protected void handleManifestException(IOException ex, Archive anArchive) throws ValidationException {
 		Logger.getLogger().logError(ex);
-		IMessage message = new Message(getBaseName(), IMessage.HIGH_SEVERITY, ERROR_READING_MANIFEST_ERROR_, new String[]{anArchive.getURI()});
+		String tmp = NLS.bind(EARValidationMessageResourceHandler.ERROR_READING_MANIFEST_ERROR_, new String[]{anArchive.getURI()});		
+		IMessage message = new LocalizedMessage(IMessage.HIGH_SEVERITY, tmp);
 		throw new ValidationException(message, ex);
 	}
 
@@ -417,7 +478,8 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 				if (uri != null && uri.indexOf(" ") != -1) { //$NON-NLS-1$
 					String[] params = new String[1];
 					params[0] = uri;
-					addError(getBaseName(), URI_CONTAINS_SPACES_ERROR_, params, appDD);
+					String tmp = NLS.bind(EARValidationMessageResourceHandler.URI_CONTAINS_SPACES_ERROR_, params);
+					addLocalizedError(tmp, appDD);
 				}// if
 			}// if
 		}// for
@@ -494,7 +556,9 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 		params[0] = projectName;
 		params[1] = earProjectName;
 		params[2] = moduleUri;
-		addError(getBaseName(), DUPLICATE_UTILJAR_FOR_PROJECT_NAME_ERROR_, params);
+		String tmp = NLS.bind(EARValidationMessageResourceHandler.DUPLICATE_UTILJAR_FOR_PROJECT_NAME_ERROR_, params);
+		
+		addLocalizedError(tmp, null);
 	}// duplicateUtilError
 
 	public void validateModuleMaps(IVirtualComponent component) {
@@ -509,12 +573,14 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 					for (int i = 0; i < modules.size(); i++) {
 						Module module = (Module) modules.get(i);
 						String uri = module.getUri();
-						IVirtualComponent referencedComp = edit.getModule( uri );
+						IVirtualComponent referencedComp = edit.getModuleByManifestURI( uri );
 						if( referencedComp == null ){
 							String[] params = new String[]{uri, component.getProject().getName()};
-							addWarning(getBaseName(), MISSING_PROJECT_FORMODULE_WARN_, params);							
+							String tmp = NLS.bind(EARValidationMessageResourceHandler.MISSING_PROJECT_FORMODULE_WARN_, params);
+							
+							addLocalizedWarning(tmp, null);							
 						}
-						//validateModuleURIExtension(module);
+						validateModuleURIExtension(module);
 					}
 				}finally{
 					if( edit != null )
@@ -532,17 +598,29 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 				String[] params = new String[1];
 				params[0] = module.getUri();
 				IResource target = earHelper.getProject().getFile(ArchiveConstants.APPLICATION_DD_URI);
-				addWarning(getBaseName(), INVALID_URI_FOR_MODULE_ERROR_, params, target);
+				String tmp = NLS.bind(EARValidationMessageResourceHandler.INVALID_URI_FOR_MODULE_ERROR_, params);
+				addLocalizedWarning(tmp, target);
 			} else if (module instanceof WebModule && !newUri.endsWith(".war")) { //$NON-NLS-1$
 				String[] params = new String[1];
 				params[0] = module.getUri();
 				IResource target = earHelper.getProject().getFile(ArchiveConstants.APPLICATION_DD_URI);
-				addWarning(getBaseName(), INVALID_URI_FOR_MODULE_ERROR_, params, target);
+				String tmp = NLS.bind(EARValidationMessageResourceHandler.INVALID_URI_FOR_MODULE_ERROR_, params);				
+				addLocalizedWarning(tmp, target);
 			}
 		}
 	}
+	
 
+	public void cleanup(IReporter reporter) {
+		if (earHelper != null)
+			earHelper.closeEARFile();
+		super.cleanup(reporter);
+	}
 
+	public void cleanup() {
+
+	}
+	
 	protected IFile getManifestFile(Archive anArchive) {
 		URIConverter conv = anArchive.getResourceSet().getURIConverter();
 		if (conv instanceof WorkbenchURIConverter) {
@@ -553,7 +631,7 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 			IProject p = input.getProject();
 			if (p == null || JemProjectUtilities.isBinaryProject(p))
 				return null;
-			IFile result = J2EEProjectUtilities.getManifestFile(p);
+			IFile result = J2EEProjectUtilities.getManifestFile(p, false);
 			if (result != null && result.exists())
 				return result;
 		}
@@ -571,13 +649,15 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 			params[0] = DOCTYPE_1_2;
 			params[1] = getResourceName();
 			params[2] = DOCTYPE_1_3;
-			addError(getBaseName(), EAR_INVALID_DOC_TYPE_ERROR_, params, appDD);
+			String tmp = NLS.bind(EARValidationMessageResourceHandler.EAR_INVALID_DOC_TYPE_ERROR_, params);			
+			addLocalizedError(tmp, appDD);
 		} else if (edit.getJ2EEVersion() < J2EEVersionConstants.J2EE_1_3_ID && appDD.getVersionID() >= J2EEVersionConstants.J2EE_1_3_ID) {
 			String[] params = new String[3];
 			params[0] = DOCTYPE_1_3;
 			params[1] = getResourceName();
 			params[2] = DOCTYPE_1_2;
-			addError(getBaseName(), EAR_INVALID_DOC_TYPE_ERROR_, params, appDD);
+			String tmp = NLS.bind(EARValidationMessageResourceHandler.EAR_INVALID_DOC_TYPE_ERROR_, params);			
+			addLocalizedError(tmp, appDD);
 		}
 	}
 
@@ -599,7 +679,8 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 						String[] params = new String[2];
 						params[0] = module.getRootFolder().getRuntimePath().toString();
 						params[1] = currentEARProject.getName();
-						addWarning(getBaseName(), URI_ALREADY_EXISTS_IN_EAR_WARN_, params, appDD);
+						String tmp = NLS.bind(EARValidationMessageResourceHandler.URI_ALREADY_EXISTS_IN_EAR_WARN_, params);						
+						addLocalizedWarning(tmp, appDD);
 					}
 				} catch (IllegalArgumentException iae) {
 					Logger.getLogger().logError(iae);
@@ -608,4 +689,13 @@ public class UIEarValidator extends EarValidator implements UIEarMessageConstant
 		}
 	}
 
+	protected void removeAllMessages(EObject eObject, String groupName) {
+		Resource res = eObject.eResource();
+		if(res != null) {
+			IFile file = WorkbenchResourceHelper.getFile(res);
+			if(file != null)
+				_reporter.removeMessageSubset(this,file,groupName);
+		}
+	}
+	
 }// UIEarValidator
