@@ -36,6 +36,10 @@ import org.eclipse.core.resources.IResourceDelta;
 import org.eclipse.core.resources.IResourceDeltaVisitor;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.Status;
+import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.emf.common.util.URI;
 import org.eclipse.emf.ecore.EObject;
 import org.eclipse.emf.ecore.resource.Resource;
@@ -43,15 +47,21 @@ import org.eclipse.jem.util.emf.workbench.ProjectUtilities;
 import org.eclipse.jem.util.emf.workbench.WorkbenchResourceHelperBase;
 import org.eclipse.jem.util.logger.proxy.Logger;
 import org.eclipse.jst.j2ee.client.ApplicationClient;
+import org.eclipse.jst.j2ee.client.ApplicationClientResource;
 import org.eclipse.jst.j2ee.ejb.EJBJar;
+import org.eclipse.jst.j2ee.ejb.EJBResource;
 import org.eclipse.jst.j2ee.ejb.EnterpriseBean;
+import org.eclipse.jst.j2ee.internal.J2EEConstants;
 import org.eclipse.jst.j2ee.internal.J2EEVersionConstants;
+import org.eclipse.jst.j2ee.internal.common.J2EEVersionUtil;
+import org.eclipse.jst.j2ee.internal.common.XMLResource;
 import org.eclipse.jst.j2ee.internal.project.J2EEProjectUtilities;
 import org.eclipse.jst.j2ee.internal.webservice.componentcore.util.WSCDDArtifactEdit;
 import org.eclipse.jst.j2ee.internal.webservice.componentcore.util.WSDDArtifactEdit;
 import org.eclipse.jst.j2ee.internal.webservices.WSDLServiceExtManager;
 import org.eclipse.jst.j2ee.internal.webservices.WSDLServiceHelper;
 import org.eclipse.jst.j2ee.webapplication.WebApp;
+import org.eclipse.jst.j2ee.webapplication.WebAppResource;
 import org.eclipse.jst.j2ee.webservice.wsclient.ComponentScopedRefs;
 import org.eclipse.jst.j2ee.webservice.wsclient.ServiceRef;
 import org.eclipse.jst.j2ee.webservice.wsclient.WebServicesClient;
@@ -65,6 +75,7 @@ import org.eclipse.wst.common.componentcore.ArtifactEdit;
 import org.eclipse.wst.common.componentcore.ComponentCore;
 import org.eclipse.wst.common.componentcore.ModuleCoreNature;
 import org.eclipse.wst.common.componentcore.internal.ArtifactEditModel;
+import org.eclipse.wst.common.componentcore.internal.util.IModuleConstants;
 import org.eclipse.wst.common.componentcore.resources.IVirtualComponent;
 import org.eclipse.wst.common.componentcore.resources.IVirtualResource;
 import org.eclipse.wst.common.internal.emfworkbench.WorkbenchResourceHelper;
@@ -79,22 +90,35 @@ import org.eclipse.wst.common.internal.emfworkbench.integration.EditModelListene
  */
 public class WebServicesManager implements EditModelListener, IResourceChangeListener, IResourceDeltaVisitor {
 
-	private HashMap wsArtifactEdits;
-	private HashMap wsClientArtifactEdits;
+	private HashMap wsArtifactEdits = new HashMap();
+	private HashMap wsClientArtifactEdits = new HashMap();
+	private List wsArtifactEditsList;
+	private List wsClientArtifactEditsList;
 	private static WebServicesManager INSTANCE = null;
 	private List listeners;
 	private List removedListeners = new ArrayList();
-	private boolean isNotifing = false;
+	private boolean isNotifying = false;
+	private boolean wsClientElementsChanged = true;
+	private boolean wsElementsChanged = true;
 
 	public static final String WSDL_EXT = "wsdl"; //$NON-NLS-1$
 	public static final String WSIL_EXT = "wsil"; //$NON-NLS-1$
 
-	public static WebServicesManager getInstance() {
+	public synchronized static WebServicesManager getInstance() {
 		if (INSTANCE == null)
 			INSTANCE = new WebServicesManager();
 		return INSTANCE;
 	}
 
+	public static  boolean isFileInteresting(IFile aFile) {
+		if (aFile != null && aFile.getFileExtension() != null) {
+			String extension = aFile.getFileExtension();
+			return extension.equals(WSDL_EXT) || extension.equals(WSIL_EXT) 
+			|| aFile.getName().equals(J2EEConstants.WEB_SERVICES_CLIENT_SHORTNAME) 
+			|| aFile.getName().equals(J2EEConstants.WEB_SERVICES_DD_URI);
+		}
+		return false;
+	}
 	/**
 	 * Default Constructor
 	 */
@@ -107,58 +131,122 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 		collectArtifactEdits();
 		ResourcesPlugin.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.POST_CHANGE);
 	}
-
-	private void collectArtifactEdits() {
-		IProject[] projects = ProjectUtilities.getAllProjects();
-		for (int i = 0; i < projects.length; i++) {
-			IProject project = projects[i];
-			WSDDArtifactEdit wsddArtifactEdit = null;
-			WSCDDArtifactEdit wscddArtifactEdit = null;
-			IVirtualComponent component = ComponentCore.createComponent(project);
-				try {
-				if (!ModuleCoreNature.isFlexibleProject(project) || J2EEProjectUtilities.isEARProject(project))
-					continue;
-				} catch (Exception e) {
-					continue;
-				}
-				//TODO implement a smarter solution than try catchs
-				try {
-				wsddArtifactEdit = WSDDArtifactEdit.getWSDDArtifactEditForRead(component);
-				if (wsddArtifactEdit != null) {
-					wsddArtifactEdit.addListener(this);
-					getWSArtifactEdits().put(component.getProject(),wsddArtifactEdit);
-				}
-				} catch (Exception e) {
-					if (wsddArtifactEdit != null) wsddArtifactEdit.dispose();
-				}
-				try {
-				wscddArtifactEdit = WSCDDArtifactEdit.getWSCDDArtifactEditForRead(component);
-				if (wscddArtifactEdit != null) {
-					wscddArtifactEdit.addListener(this);
-					getWSClientArtifactEdits().put(component.getProject(),wscddArtifactEdit);
-				}
-				} catch (Exception e) {
-					if (wscddArtifactEdit != null) wscddArtifactEdit.dispose();
+	
+	private void addArtifactEdit(IProject handle) {
+		synchronized(wsArtifactEdits) {
+			if (!wsArtifactEdits.containsKey(handle)) {
+				WSDDArtifactEdit edit = WSDDArtifactEdit.getWSDDArtifactEditForRead(handle);
+				if (edit != null) {
+					WebServices webServices = edit.getWebServices();
+					// If the project has a webservice.xml with internal services or 
+					// it has .wsil files with external services, we cache the artifact edit
+					if ((webServices != null && !webServices.getWebServiceDescriptions().isEmpty()) || !edit.getWSILResources().isEmpty()) {
+						edit.addListener(this);
+						wsArtifactEdits.put(handle, edit);
+						wsElementsChanged = true;
+					// Otherwise, dispose the artifact edit
+					} else {
+						edit.dispose();
+					}
 				}
 			}
+		}
+		synchronized (wsClientArtifactEdits) {
+			if (!wsClientArtifactEdits.containsKey(handle)) {
+				WSCDDArtifactEdit edit = WSCDDArtifactEdit.getWSCDDArtifactEditForRead(handle);
+				if (edit != null) {
+					WebServicesResource res = edit.getWscddXmiResource();
+					boolean isInterested = false;
+					// Does the project have 1.3 web service clients?
+					if (res != null && res.isLoaded() && res.getWebServicesClient() != null)
+						isInterested = true;
+					
+					// Does the project have 1.4 web service clients?
+					List wsClientEdits = new ArrayList();
+					wsClientEdits.add(edit);
+					//if (!getWorkspace14ServiceRefs(wsClientEdits).isEmpty())
+						isInterested = true;
+					
+					// If project has 1.3 or 1.4 web service clients, cache the artifact edit
+					if (isInterested) {
+						edit.addListener(this);
+						wsClientArtifactEdits.put(handle, edit);
+						wsClientElementsChanged = true;
+					// Otherwise, dispose the artifact edit	
+					} else {
+						edit.dispose();
+					}
+				}
+			}
+		}
+	}
+	
+	private void removeArtifactEdit(IProject handle) {
+		synchronized(wsArtifactEdits) {
+			if (wsArtifactEdits.containsKey(handle)) {
+				ArtifactEdit edit = (ArtifactEdit) wsArtifactEdits.get(handle);
+				if (edit != null) {
+					wsArtifactEdits.remove(handle);
+					edit.removeListener(this);
+					edit.dispose();
+					wsElementsChanged = true;
+				}
+			}
+		}
+		synchronized (wsClientArtifactEdits) {
+			if (wsClientArtifactEdits.containsKey(handle)) {
+				ArtifactEdit edit = (ArtifactEdit) wsClientArtifactEdits.get(handle);
+				if (edit != null) {
+					wsClientArtifactEdits.remove(handle);
+					edit.removeListener(this);
+					edit.dispose();
+					wsClientElementsChanged = true;
+				}
+			}
+		}
+	}
+
+	private void collectArtifactEdits() {
+		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
+		for (int i = 0; i < projects.length; i++) {
+			IProject project = projects[i];
+			IVirtualComponent component = ComponentCore.createComponent(project);
+			try {
+				if (!ModuleCoreNature.isFlexibleProject(project) || J2EEProjectUtilities.isEARProject(project) || J2EEProjectUtilities.isStaticWebProject(project))
+					continue;
+			} catch (Exception e) {
+				continue;
+			}
+			addArtifactEdit(component.getProject());
+		}
 	}
 
 	/**
-	 * @return Returns the editModels.
+	 * @return Returns the artifact edit iterator for web service artifact edits
 	 */
-	private HashMap getWSArtifactEdits() {
-		if (wsArtifactEdits == null)
-			wsArtifactEdits = new HashMap();
-		return wsArtifactEdits;
+	private List getWSArtifactEdits() {
+		synchronized (wsArtifactEdits) {
+			if (wsElementsChanged) {
+				wsArtifactEditsList = new ArrayList();
+				wsArtifactEditsList.addAll(wsArtifactEdits.values());
+				wsElementsChanged = false;
+			}
+		}
+		return wsArtifactEditsList;
 	}
 	
 	/**
 	 * @return Returns the editModels.
 	 */
-	private HashMap getWSClientArtifactEdits() {
-		if (wsClientArtifactEdits == null)
-			wsClientArtifactEdits = new HashMap();
-		return wsClientArtifactEdits;
+	private List getWSClientArtifactEdits() {
+		synchronized (wsClientArtifactEdits) {
+			if (wsClientElementsChanged) {
+				wsClientArtifactEditsList = new ArrayList();
+				wsClientArtifactEditsList.addAll(wsClientArtifactEdits.values());
+				wsClientElementsChanged = false;
+			}
+		}
+		return wsClientArtifactEditsList;
 	}
 
 	private List getListeners() {
@@ -187,61 +275,19 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 			List resources = anEvent.getChangedResources();
 			for (int i=0; i<resources.size(); i++) {
 				Resource res = (Resource) resources.get(i);
-				if (res instanceof WsddResource || res instanceof WebServicesResource) {
+				if (isInterestedInResource(res))
 					notifyListeners(anEvent.getEventCode());
-				}
 			}
 		}
 		else if (anEvent.getEventCode() == EditModelEvent.PRE_DISPOSE) {
 			ArtifactEditModel editModel = (ArtifactEditModel) anEvent.getEditModel();
 			if (editModel == null || editModel.getProject() == null)
 				return;
-			WSDDArtifactEdit wsArtifactEdit = (WSDDArtifactEdit) getWSArtifactEdits().get(editModel.getProject());
-			if (wsArtifactEdit != null) {
-				try {
-					getWSArtifactEdits().remove(editModel.getProject());
-					wsArtifactEdit.removeListener(this);
-				} finally {
-					wsArtifactEdit.dispose();
-				}
-			}
-			WSCDDArtifactEdit wsClientArtifactEdit = (WSCDDArtifactEdit) getWSClientArtifactEdits().get(editModel.getProject());
-			if (wsClientArtifactEdit != null) {
-				try {
-					getWSClientArtifactEdits().remove(editModel.getProject());
-					wsClientArtifactEdit.removeListener(this);
-				} finally {
-				wsClientArtifactEdit.dispose();
-				}
-			}
+			removeArtifactEdit(editModel.getProject());
 			notifyListeners(anEvent.getEventCode());
 		}
 	}
-	
-	private WSDDArtifactEdit getWSArtifactEdit(IProject handle) {
-		WSDDArtifactEdit artifactEdit = (WSDDArtifactEdit) getWSArtifactEdits().get(handle);
-		if (artifactEdit == null) {
-			artifactEdit = WSDDArtifactEdit.getWSDDArtifactEditForRead(handle);
-			if (artifactEdit != null) {
-				artifactEdit.addListener(this);
-				getWSArtifactEdits().put(handle,artifactEdit);
-			}
-		}
-		return artifactEdit;
-	}
-	
-	private WSCDDArtifactEdit getWSClientArtifactEdit(IProject handle) {
-		WSCDDArtifactEdit artifactEdit = (WSCDDArtifactEdit) getWSClientArtifactEdits().get(handle);
-		if (artifactEdit == null) {
-			artifactEdit = WSCDDArtifactEdit.getWSCDDArtifactEditForRead(handle);
-			if (artifactEdit != null) {
-				artifactEdit.addListener(this);
-				getWSClientArtifactEdits().put(handle,artifactEdit);
-			}
-		}
-	
-	return artifactEdit;
-	}
+
 
 	/**
 	 * Notify listeners of
@@ -252,17 +298,30 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 		if (listeners == null)
 			return;
 		synchronized (this) {
-			isNotifing = true;
+			isNotifying = true;
 		}
 		try {
+			WebServiceEvent webServiceEvent = null;
+			
+			switch (anEventType) {
+			case EditModelEvent.UNLOADED_RESOURCE:
+			case EditModelEvent.PRE_DISPOSE:
+			case EditModelEvent.REMOVED_RESOURCE:
+				webServiceEvent = new WebServiceEvent(WebServiceEvent.REMOVE);
+				break;
+			default:
+				if (!getAllWorkspaceServiceRefs().isEmpty() || !getAllWSDLServices().isEmpty())
+					webServiceEvent = new WebServiceEvent(WebServiceEvent.REFRESH);
+			}
 			List list = getListeners();
-			for (int i = 0; i < list.size(); i++) {
-				WebServiceEvent webServiceEvent = new WebServiceEvent(WebServiceEvent.REFRESH);
-				((WebServiceManagerListener) list.get(i)).webServiceManagerChanged(webServiceEvent);
+			if (webServiceEvent!=null) {
+				for (int i = 0; i < list.size(); i++) {
+					((WebServiceManagerListener) list.get(i)).webServiceManagerChanged(webServiceEvent);
+				}
 			}
 		} finally {
 			synchronized (this) {
-				isNotifing = false;
+				isNotifying = false;
 				if (removedListeners != null && !removedListeners.isEmpty()) {
 					for (int i = 0; i < removedListeners.size(); i++)
 						listeners.remove(removedListeners.get(i));
@@ -277,7 +336,7 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 	 */
 	public synchronized boolean removeListener(WebServiceManagerListener aListener) {
 		if (aListener != null) {
-			if (isNotifing)
+			if (isNotifying)
 				return removedListeners.add(aListener);
 			return getListeners().remove(aListener);
 		}
@@ -285,21 +344,25 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 	}
 
 	private void releaseArtifactEdits() {
-		Iterator iter = getWSArtifactEdits().values().iterator();
-		while (iter.hasNext()) {
-			WSDDArtifactEdit artifactEdit = (WSDDArtifactEdit) iter.next();
-			artifactEdit.removeListener(this);
-			artifactEdit.dispose();
+		synchronized (wsArtifactEdits) {
+			Iterator iter = wsArtifactEdits.values().iterator();
+			while (iter.hasNext()) {
+				WSDDArtifactEdit artifactEdit = (WSDDArtifactEdit) iter.next();
+				artifactEdit.removeListener(this);
+				artifactEdit.dispose();
+			}
+			wsArtifactEdits.clear();
 		}
-		getWSArtifactEdits().clear();
 		
-		iter = getWSClientArtifactEdits().values().iterator();
-		while (iter.hasNext()) {
-			WSCDDArtifactEdit artifactEdit = (WSCDDArtifactEdit) iter.next();
-			artifactEdit.removeListener(this);
-			artifactEdit.dispose();
+		synchronized (wsClientArtifactEdits) {
+			Iterator iter = wsClientArtifactEdits.values().iterator();
+			while (iter.hasNext()) {
+				WSCDDArtifactEdit artifactEdit = (WSCDDArtifactEdit) iter.next();
+				artifactEdit.removeListener(this);
+				artifactEdit.dispose();
+			}
+			wsClientArtifactEdits.clear();
 		}
-		getWSClientArtifactEdits().clear();
 	}
 
 	/*
@@ -343,7 +406,7 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 	 */
 	public List getInternalWebServices() {
 		List result = new ArrayList();
-		Iterator iter = getWSArtifactEdits().values().iterator();
+		Iterator iter = getWSArtifactEdits().iterator();
 		while (iter.hasNext()) {
 			WSDDArtifactEdit artifactEdit = (WSDDArtifactEdit) iter.next();
 			WebServices webServices = artifactEdit.getWebServices();
@@ -450,12 +513,32 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 		String wsdlFileName = webService.getWsdlFile();
 		Resource res = null;
 		IVirtualResource[] resources = ComponentCore.createResources(WorkbenchResourceHelper.getFile(webService));
-		if (resources == null) return res;
-		WSDDArtifactEdit artifactEdit = (WSDDArtifactEdit) getWSArtifactEdits().get(resources[0].getComponent().getProject());
+		if (resources == null || resources.length==0) return res;
+		WSDDArtifactEdit artifactEdit = getExistingWSDDArtifactEdit(resources[0].getComponent().getProject());
 		if (artifactEdit!=null) 
 			res = artifactEdit.getWsdlResource(wsdlFileName);
 		return res;
 	}
+	
+	private WSDDArtifactEdit getExistingWSDDArtifactEdit(IProject project) {
+		List wsEdits = getWSArtifactEdits();
+		for (int i=0; i<wsEdits.size(); i++) {
+			WSDDArtifactEdit edit = (WSDDArtifactEdit) wsEdits.get(i);
+			if (edit !=null && edit.getProject()!= null && edit.getProject().equals(project))
+				return edit;
+		}
+		return null;
+	}
+	
+	private WSCDDArtifactEdit getExistingWSCDDArtifactEdit(IProject project) {
+		List wsClientEdits = getWSClientArtifactEdits();
+		for (int i=0; i<wsClientEdits.size(); i++) {
+			WSCDDArtifactEdit edit = (WSCDDArtifactEdit) wsClientEdits.get(i);
+			if (edit !=null && edit.getProject()!= null && edit.getProject().equals(project))
+				return edit;
+		}
+		return null;
+	}
 
 	public List getWSDLServices(Resource wsdl) {
 		WSDLServiceHelper serviceHelper = WSDLServiceExtManager.getServiceHelper();
@@ -575,20 +658,26 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 
 	public List getWorkspace13ServiceRefs() {
 		List result = new ArrayList();
-		Iterator iter = getWSClientArtifactEdits().values().iterator();
+		Iterator iter = getWSClientArtifactEdits().iterator();
 		while (iter.hasNext()) {
 			WSCDDArtifactEdit artifactEdit = (WSCDDArtifactEdit) iter.next();
 			WebServicesResource res = artifactEdit.getWscddXmiResource();
-			if (res != null && res.isLoaded() && res.getWebServicesClient() != null)
-				result.addAll(res.getWebServicesClient().getServiceRefs());
+			if (res != null && res.isLoaded() && res.getWebServicesClient() != null) {
+				if (J2EEProjectUtilities.isEJBProject(artifactEdit.getProject())) {
+					List scopedBeans = res.getWebServicesClient().getComponentScopedRefs();
+					for (Iterator iterator = scopedBeans.iterator(); iterator.hasNext();) {
+						ComponentScopedRefs refBean = (ComponentScopedRefs) iterator.next();
+						result.addAll(refBean.getServiceRefs());
+					}
+				} else	result.addAll(res.getWebServicesClient().getServiceRefs());
+			}
 		}
 		return result;
 	}
 
 	public List get13ServiceRefs(IProject handle) {
-
 		List result = new ArrayList();
-		WSCDDArtifactEdit wsClientArtifactEdit = (WSCDDArtifactEdit) getWSClientArtifactEdits().get(handle);
+		WSCDDArtifactEdit wsClientArtifactEdit = getExistingWSCDDArtifactEdit(handle);
 		if (wsClientArtifactEdit !=null) {
 			WebServicesResource res = wsClientArtifactEdit.getWscddXmiResource();
 			if (res != null && res.isLoaded() && res.getWebServicesClient() != null)
@@ -598,13 +687,37 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 	}
 
 	public List getWorkspace14ServiceRefs() {
+		return getWorkspace14ServiceRefs(getWSClientArtifactEdits());
+	}
+	
+	private List getWorkspace14ServiceRefs(List wsClientArtifactEdits) {
+		Iterator iter = wsClientArtifactEdits.iterator();
 		List result = new ArrayList();
-		Iterator iter = getWSClientArtifactEdits().values().iterator();
 		while (iter.hasNext()) {
 			WSCDDArtifactEdit wscArtifactEdit = (WSCDDArtifactEdit) iter.next();
-			ArtifactEdit artifactEdit = ArtifactEdit.getArtifactEditForRead(wscArtifactEdit.getProject());
+
+			IProject project = wscArtifactEdit.getProject(); 	
+			
+			String projectType = J2EEProjectUtilities.getJ2EEProjectType(project);
+			String projectVersion = J2EEProjectUtilities.getJ2EEProjectVersion(project);
+			int    j2eeLevel =  0;
+			if(IModuleConstants.JST_EJB_MODULE.equals(projectType)){
+				j2eeLevel = J2EEVersionUtil.convertEJBVersionStringToJ2EEVersionID(projectVersion);
+			} else if(IModuleConstants.JST_WEB_MODULE.equals(projectType)){
+				j2eeLevel = J2EEVersionUtil.convertWebVersionStringToJ2EEVersionID(projectVersion);
+			} else if(IModuleConstants.JST_APPCLIENT_MODULE.equals(projectType)){
+				j2eeLevel = J2EEVersionUtil.convertAppClientVersionStringToJ2EEVersionID(projectVersion);
+			}
+			
+			// this method needs to check that project's j2ee level is 14
+			if(j2eeLevel !=  J2EEVersionConstants.J2EE_1_4_ID)
+				continue;
+			
+			ArtifactEdit artifactEdit = ArtifactEdit.getArtifactEditForRead(project);
 			try {
-				EObject rootObject = artifactEdit.getContentModelRoot();
+				EObject rootObject = null;
+				if (artifactEdit!=null)
+					rootObject = artifactEdit.getContentModelRoot();
 				// handle EJB project case
 				if (rootObject instanceof EJBJar) {
 					List cmps = ((EJBJar) rootObject).getEnterpriseBeans();
@@ -638,7 +751,7 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 
 	public List getWorkspaceWSILFiles() {
 		List result = new ArrayList();
-		Iterator iter = getWSArtifactEdits().values().iterator();
+		Iterator iter = getWSArtifactEdits().iterator();
 		while (iter.hasNext()) {
 			WSDDArtifactEdit artifactEdit = (WSDDArtifactEdit) iter.next();
 			List files = artifactEdit.getWSILResources();
@@ -653,7 +766,7 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 
 	public List getWorkspaceWSDLResources() {
 		List result = new ArrayList();
-		Iterator iter = getWSArtifactEdits().values().iterator();
+		Iterator iter = getWSArtifactEdits().iterator();
 		while (iter.hasNext()) {
 			WSDDArtifactEdit artifactEdit = (WSDDArtifactEdit) iter.next();
 			IProject project = artifactEdit.getProject();
@@ -685,7 +798,10 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 	 * @return
 	 */
 	public List get13ServiceRefs(EnterpriseBean bean) {
-		WSCDDArtifactEdit artifactEdit = (WSCDDArtifactEdit) getWSClientArtifactEdits().get(WorkbenchResourceHelper.getFile(bean).getProject());
+		WSCDDArtifactEdit artifactEdit = null;
+		IFile file = WorkbenchResourceHelper.getFile(bean);
+		if (file!=null)
+			artifactEdit = getExistingWSCDDArtifactEdit(file.getProject());
 		if (artifactEdit !=null) {
 			WebServicesResource res = artifactEdit.getWscddXmiResource();
 			if (res != null && res.getWebServicesClient() != null) {
@@ -709,7 +825,7 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 		IProject handle = getComponentProject(WorkbenchResourceHelper.getFile(client));
 		if (handle == null)
 			return Collections.EMPTY_LIST;
-		WSCDDArtifactEdit artifactEdit = (WSCDDArtifactEdit) getWSClientArtifactEdits().get(handle);
+		WSCDDArtifactEdit artifactEdit = getExistingWSCDDArtifactEdit(handle);
 		if (artifactEdit !=null) {
 			WebServicesResource res = artifactEdit.getWscddXmiResource();
 			if (res != null) {
@@ -729,7 +845,7 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 		IProject handle = getComponentProject(WorkbenchResourceHelper.getFile(webapp));
 		if (handle == null)
 			return Collections.EMPTY_LIST;
-		WSCDDArtifactEdit artifactEdit = (WSCDDArtifactEdit) getWSClientArtifactEdits().get(handle);
+		WSCDDArtifactEdit artifactEdit = getExistingWSCDDArtifactEdit(handle);
 		if (artifactEdit !=null) {
 			WebServicesResource res = artifactEdit.getWscddXmiResource();
 			if (res != null) {
@@ -752,55 +868,36 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 			IProject p = (IProject) resource;
 			// Handle project adds and project opens
 			if ((delta.getKind()==IResourceDelta.ADDED || (((delta.getFlags() & IResourceDelta.OPEN) != 0) && p.isAccessible()))) {
-				boolean state = true;
 				IVirtualComponent component = ComponentCore.createComponent(p);
-				if (component!=null && !J2EEProjectUtilities.isEARProject(p)) {
-					WSDDArtifactEdit wsArtifactEdit = getWSArtifactEdit(p);
-					if (wsArtifactEdit !=null)
-						state = false;
-					WSCDDArtifactEdit wscArtifactEdit = getWSClientArtifactEdit(p);
-					if (wscArtifactEdit !=null) {
-						state = false;
-					}
-				return state;
+				if (component!=null && !J2EEProjectUtilities.isEARProject(p) && !J2EEProjectUtilities.isStaticWebProject(p)) {
+					Job job = new ProcessProjectsWithWSDL(p, EditModelEvent.ADDED_RESOURCE);
+					job.setRule(p);
+					job.schedule();
+					return false;
 				}
 			}
 			// Handle project close events and removals 
 			else if ((delta.getKind() == IResourceDelta.CHANGED  && ((delta.getFlags() & IResourceDelta.OPEN) != 0))
 					|| (delta.getKind() == IResourceDelta.REMOVED)){
-				boolean state = true;
-				List wsddArtifactEditsToRemove = getAssociatedArtifactEditKeys(p,getWSArtifactEdits());
-				for (int i=0; i<wsddArtifactEditsToRemove.size(); i++) {
-					IProject handle = (IProject) wsddArtifactEditsToRemove.get(i);
-					if (handle != null && getWSArtifactEdits().containsKey(handle)) {
-						WSDDArtifactEdit wsArtifactEdit = (WSDDArtifactEdit) getWSArtifactEdits().get(handle);
-						getWSArtifactEdits().remove(handle);
-						wsArtifactEdit.dispose();
-						state = false;
-					}
-				}
-				List wscddArtifactEditsToRemove = getAssociatedArtifactEditKeys(p,getWSClientArtifactEdits());
-				for (int i=0; i<wscddArtifactEditsToRemove.size(); i++) {
-					IProject handle = (IProject) wscddArtifactEditsToRemove.get(i);
-					if (handle != null && getWSClientArtifactEdits().containsKey(handle)) {
-						WSCDDArtifactEdit wscArtifactEdit = (WSCDDArtifactEdit) getWSClientArtifactEdits().get(handle);
-						getWSClientArtifactEdits().remove(handle);
-						wscArtifactEdit.dispose();
-						state = false;
-					}	
-				}
-				return state;
+				removeArtifactEdit(p);
+				return false;
 			}
 		}
 		
 		else if (resource.getType() == IResource.FILE && isInterrestedInFile((IFile) resource)) {
-			// Handle WSIL and WSDL File additions
+			// Handle WSIL and WSDL File additions as well as webservice.xml and webserviceclient.xml
 			if ((delta.getKind() == IResourceDelta.ADDED) || ((delta.getFlags() & IResourceDelta.MOVED_TO) != 0)) {
 				if (resource.getFileExtension().equals(WSDL_EXT))
 				    addedWsdl((IFile) resource);
 				else if (resource.getFileExtension().equals(WSIL_EXT))
 				    addedWsil((IFile)resource);
-			}
+				else if (resource.getName().equals(J2EEConstants.WEB_SERVICES_CLIENT_SHORTNAME) ||
+						resource.getName().equals(J2EEConstants.WEB_SERVICES_DD_URI)) {
+					Job job = new ProcessProjectsWithWSDL(resource.getProject(), EditModelEvent.LOADED_RESOURCE);
+					job.setRule(resource.getProject());
+					job.schedule();
+				}	
+			} 
 			// Handle WSIL or WSDL file removals
 			else if ((delta.getKind() == IResourceDelta.REMOVED) || ((delta.getFlags() & IResourceDelta.MOVED_FROM) != 0)) {
 				if (resource.getFileExtension().equals(WSDL_EXT) || resource.getFileExtension().equals(WSIL_EXT))
@@ -810,17 +907,24 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 		}
 		return true;
 	}
-	
-	private List getAssociatedArtifactEditKeys(IProject project, HashMap artifactEdits) {
-		List result = new ArrayList();
-		Iterator iter = artifactEdits.entrySet().iterator();
-		while (iter.hasNext()) {
-			Map.Entry entry = (Map.Entry) iter.next();
-			IProject handle = (IProject) entry.getKey();
-			if (handle.getProject().equals(project))
-				result.add(handle);
+
+	private class ProcessProjectsWithWSDL extends Job
+	{
+		private IProject currentProject;
+		private int eventType;
+		
+		public ProcessProjectsWithWSDL(IProject p, int newEventType)
+		{
+			super("Loading artifact edit for project");
+			currentProject = p;
+			eventType = newEventType;
+		}
+		
+		protected IStatus run(IProgressMonitor monitor) {
+			addArtifactEdit(currentProject);
+			notifyListeners(eventType);
+			return Status.OK_STATUS;
 		}
-		return result;
 	}
 
 	protected void addedWsdl(IFile wsdl) {
@@ -828,7 +932,7 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 			return;
 		IProject handle = getComponentProject(wsdl);
 		if (handle != null) {
-			getWSArtifactEdit(handle);
+			addArtifactEdit(handle);
 			notifyListeners(EditModelEvent.LOADED_RESOURCE);
 		}
 	}
@@ -838,21 +942,35 @@ public class WebServicesManager implements EditModelListener, IResourceChangeLis
 			return;
 		IProject handle = getComponentProject(wsil);
 		if (handle != null) {
-			getWSArtifactEdit(handle);
+			addArtifactEdit(handle);
 			notifyListeners(EditModelEvent.LOADED_RESOURCE);
 		}
 	}
 	
 	private IProject getComponentProject(IFile res) {
+		if (res==null)
+			return null;
 		return res.getProject();
 	}
 
 	protected boolean isInterrestedInFile(IFile aFile) {
-		if (aFile != null && aFile.getFileExtension() != null) {
-			String extension = aFile.getFileExtension();
-			return extension.equals(WSDL_EXT) || extension.equals(WSIL_EXT);
+		return isFileInteresting(aFile);
+	}
+	
+	/**
+	 * The only resources webservice manager is concerned about are:
+	 * webservice.xml, webserviceclient.xml, and J2EE 1.4 web.xml, ejb-jar-xml, and application-client.xml
+	 * @param res
+	 * @return boolean isInterested
+	 */
+	private boolean isInterestedInResource(Resource res) {
+		if (res instanceof WsddResource || res instanceof WebServicesResource) {
+			return true;
+		} else if (res instanceof EJBResource || res instanceof WebAppResource || res instanceof ApplicationClientResource) {
+			return ((XMLResource)res).getJ2EEVersionID()>J2EEVersionConstants.J2EE_1_3_ID;
+		} else {
+			return false;
 		}
-		return false;
 	}
 
 	/**
