@@ -10,10 +10,10 @@
  *******************************************************************************/
 package org.eclipse.jst.j2ee.jca.project.facet;
 
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
 import java.lang.reflect.InvocationTargetException;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
 
 import org.eclipse.core.commands.ExecutionException;
 import org.eclipse.core.resources.IFile;
@@ -24,44 +24,41 @@ import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.core.runtime.Path;
-import org.eclipse.emf.ecore.EPackage;
 import org.eclipse.jdt.core.IClasspathEntry;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaCore;
-import org.eclipse.jem.util.emf.workbench.ProjectUtilities;
+import org.eclipse.jem.util.emf.workbench.WorkbenchByteArrayOutputStream;
 import org.eclipse.jem.util.logger.proxy.Logger;
 import org.eclipse.jst.common.project.facet.WtpUtils;
 import org.eclipse.jst.common.project.facet.core.ClasspathHelper;
-import org.eclipse.jst.j2ee.application.ApplicationPackage;
-import org.eclipse.jst.j2ee.application.Module;
-import org.eclipse.jst.j2ee.application.internal.operations.AddComponentToEnterpriseApplicationDataModelProvider;
-import org.eclipse.jst.j2ee.application.internal.operations.AddComponentToEnterpriseApplicationOp;
-import org.eclipse.jst.j2ee.application.internal.operations.IAddComponentToEnterpriseApplicationDataModelProperties;
+import org.eclipse.jst.j2ee.commonarchivecore.internal.util.ArchiveUtil;
 import org.eclipse.jst.j2ee.internal.J2EEConstants;
+import org.eclipse.jst.j2ee.internal.J2EEVersionConstants;
 import org.eclipse.jst.j2ee.internal.common.J2EEVersionUtil;
+import org.eclipse.jst.j2ee.internal.common.classpath.J2EEComponentClasspathContainer;
+import org.eclipse.jst.j2ee.internal.common.classpath.J2EEComponentClasspathContainerUtils;
+import org.eclipse.jst.j2ee.jca.Connector;
 import org.eclipse.jst.j2ee.jca.modulecore.util.ConnectorArtifactEdit;
-import org.eclipse.jst.j2ee.project.facet.IJ2EEFacetInstallDataModelProperties;
 import org.eclipse.jst.j2ee.project.facet.IJ2EEModuleFacetInstallDataModelProperties;
 import org.eclipse.jst.j2ee.project.facet.J2EEFacetInstallDelegate;
 import org.eclipse.wst.common.componentcore.ComponentCore;
 import org.eclipse.wst.common.componentcore.datamodel.FacetDataModelProvider;
-import org.eclipse.wst.common.componentcore.datamodel.properties.ICreateReferenceComponentsDataModelProperties;
 import org.eclipse.wst.common.componentcore.datamodel.properties.IFacetDataModelProperties;
 import org.eclipse.wst.common.componentcore.resources.IVirtualComponent;
 import org.eclipse.wst.common.componentcore.resources.IVirtualFile;
 import org.eclipse.wst.common.componentcore.resources.IVirtualFolder;
-import org.eclipse.wst.common.frameworks.datamodel.DataModelFactory;
 import org.eclipse.wst.common.frameworks.datamodel.IDataModel;
 import org.eclipse.wst.common.frameworks.datamodel.IDataModelOperation;
 import org.eclipse.wst.common.project.facet.core.IDelegate;
-import org.eclipse.wst.common.project.facet.core.IFacetedProject;
 import org.eclipse.wst.common.project.facet.core.IProjectFacetVersion;
-import org.eclipse.wst.common.project.facet.core.ProjectFacetsManager;
-import org.eclipse.wst.common.project.facet.core.runtime.IRuntime;
 
 public class ConnectorFacetInstallDelegate extends J2EEFacetInstallDelegate implements IDelegate {
 
+	public static final String CONNECTOR_XML_TEMPLATE_10		= "rartp10.xml";								//$NON-NLS-1$
+	public static final String CONNECTOR_XML_TEMPLATE_15		= "rartp15.xml"; //$NON-NLS-1$
+	
 	public void execute(IProject project, IProjectFacetVersion fv, Object config, IProgressMonitor monitor) throws CoreException {
 		if (monitor != null) {
 			monitor.beginTask("", 1); //$NON-NLS-1$
@@ -86,7 +83,7 @@ public class ConnectorFacetInstallDelegate extends J2EEFacetInstallDelegate impl
 			final IVirtualComponent c = ComponentCore.createComponent(project);
 
 			c.create(0, null);
-			c.setMetaProperty("java-output-path", "/build/classes/"); //$NON-NLS-1$ //$NON-NLS-2$
+			setOutputFolder(model, c);
 
 			final IVirtualFolder root = c.getRootFolder();
 			IFolder sourceFolder = null;
@@ -100,7 +97,46 @@ public class ConnectorFacetInstallDelegate extends J2EEFacetInstallDelegate impl
 			if (!sourceFolder.getFile(J2EEConstants.RAR_DD_URI).exists()) {
 				String ver = model.getStringProperty(IFacetDataModelProperties.FACET_VERSION_STR);
 				int nVer = J2EEVersionUtil.convertVersionStringToInt(ver);
-				ConnectorArtifactEdit.createDeploymentDescriptor(project, nVer);
+				
+				IFile aFile = sourceFolder.getFile(new Path(J2EEConstants.RAR_DD_URI));
+				OutputStream out = new WorkbenchByteArrayOutputStream(aFile);
+				String template = nVer == J2EEVersionConstants.JCA_1_0_ID
+						? CONNECTOR_XML_TEMPLATE_10
+						: CONNECTOR_XML_TEMPLATE_15;
+				InputStream in = getClass().getResourceAsStream(template);
+				if (in != null & out != null) {
+					try {
+						ArchiveUtil.copy(in, out);
+					} catch (IOException ioe) {
+						Logger.getLogger().logError(ioe);
+					} finally{
+						try{
+							if(null != out){
+								out.close();
+							} if(null != in){
+								in.close();
+							}
+						}catch (IOException ioe) {
+							Logger.getLogger().logError(ioe);
+						} 
+					}
+					
+					ConnectorArtifactEdit edit = new ConnectorArtifactEdit(project, false, true);
+					try{
+						if( edit != null ){
+							Connector connector = edit.getConnector();
+							connector.setDisplayName(project.getName());
+							edit.saveIfNecessary(new NullProgressMonitor());
+						}
+					}finally{
+						if( edit != null ){
+							edit.dispose();
+						}
+					}
+				} else {
+					//without a template
+					ConnectorArtifactEdit.createDeploymentDescriptor(project, nVer);
+				} // if				
 			}
 			
 			// add source folder maps
@@ -132,51 +168,9 @@ public class ConnectorFacetInstallDelegate extends J2EEFacetInstallDelegate impl
 				// ClasspathHelper.addClasspathEntries( project, fv, <something> );
 			}
 
-			// Associate with an EAR, if necessary.
-
-			final String earProjectName = (String) model.getProperty(IJ2EEModuleFacetInstallDataModelProperties.EAR_PROJECT_NAME);
-			if (model.getBooleanProperty(IJ2EEModuleFacetInstallDataModelProperties.ADD_TO_EAR)) {
-				if (earProjectName != null && !earProjectName.equals("")) { //$NON-NLS-1$
-
-					String ver = fv.getVersionString();
-					String j2eeVersionText = J2EEVersionUtil.convertVersionIntToString
-					(J2EEVersionUtil.convertConnectorVersionStringToJ2EEVersionID(ver));
-					
-					IFacetedProject facetedProject = ProjectFacetsManager.create(project);
-					installEARFacet(j2eeVersionText, earProjectName, (IRuntime) model.getProperty(IJ2EEFacetInstallDataModelProperties.FACET_RUNTIME), monitor);
-
-					IProject earProject = ProjectUtilities.getProject(earProjectName);
-					IVirtualComponent earComp = ComponentCore.createComponent(earProject);
-					final String moduleURI = model.getStringProperty(IJ2EEModuleFacetInstallDataModelProperties.MODULE_URI);
-
-					final IDataModel dataModel = DataModelFactory.createDataModel(new AddComponentToEnterpriseApplicationDataModelProvider() {
-						public Object getDefaultProperty(String propertyName) {
-							if (IAddComponentToEnterpriseApplicationDataModelProperties.TARGET_COMPONENTS_TO_URI_MAP.equals(propertyName)) {
-								Map map = new HashMap();
-								map.put(c, moduleURI);
-								return map;
-							}
-							return super.getDefaultProperty(propertyName);
-						}
-
-						public IDataModelOperation getDefaultOperation() {
-							return new AddComponentToEnterpriseApplicationOp(model) {
-								protected Module createNewModule(IVirtualComponent wc) {
-									return ((ApplicationPackage) EPackage.Registry.INSTANCE.getEPackage(ApplicationPackage.eNS_URI)).getApplicationFactory().createConnectorModule();
-								}
-							};
-						}
-					});
-					dataModel.setProperty(ICreateReferenceComponentsDataModelProperties.SOURCE_COMPONENT, earComp);
-					List modList = (List) dataModel.getProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST);
-					modList.add(c);
-					dataModel.setProperty(ICreateReferenceComponentsDataModelProperties.TARGET_COMPONENT_LIST, modList);
-					try {
-						dataModel.getDefaultOperation().execute(null, null);
-					} catch (ExecutionException e) {
-						Logger.getLogger().logError(e);
-					}
-				}
+			if(J2EEComponentClasspathContainerUtils.getDefaultUseEARLibraries()){
+				final IPath earLibContainer = new Path(J2EEComponentClasspathContainer.CONTAINER_ID);
+				addToClasspath(jproj, JavaCore.newContainerEntry(earLibContainer));
 			}
 
 			try {
