@@ -19,6 +19,9 @@ import java.util.Map;
 import java.util.Set;
 import java.util.logging.Level;
 
+import org.eclipse.core.resources.IProject;
+import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.jobs.ISchedulingRule;
 import org.eclipse.jem.java.JavaClass;
 import org.eclipse.jem.util.logger.LogEntry;
 import org.eclipse.jem.util.logger.proxy.Logger;
@@ -27,10 +30,10 @@ import org.eclipse.jst.j2ee.ejb.EnterpriseBean;
 import org.eclipse.jst.j2ee.ejb.Entity;
 import org.eclipse.jst.j2ee.internal.J2EEConstants;
 import org.eclipse.wst.validation.internal.core.ValidationException;
+import org.eclipse.wst.validation.internal.operations.WorkbenchReporter;
 import org.eclipse.wst.validation.internal.provisional.core.IMessage;
 import org.eclipse.wst.validation.internal.provisional.core.IReporter;
 import org.eclipse.wst.validation.internal.provisional.core.IValidationContext;
-import org.eclipse.wst.validation.internal.provisional.core.MessageLimitException;
 
 /**
  * @version 	1.0
@@ -60,10 +63,7 @@ public class EJBValidator extends AbstractEJBValidator {
 		return "ejbvalidator"; //$NON-NLS-1$
 	}
 
-	/*
-	 * @see IValidator#validate(IValidationContext, IReporter, IFileDelta[])
-	 */
-	public void validate(IValidationContext helper, IReporter reporter) throws ValidationException {
+	public void commonValidate(IValidationContext helper, IReporter reporter) throws ValidationException {
 		long start = System.currentTimeMillis();
 		Logger logger = Logger.getLogger(IEJBValidatorConstants.J2EE_CORE_PLUGIN);
 		if (logger != null && logger.isLoggingLevel(Level.FINER)) {
@@ -89,11 +89,20 @@ public class EJBValidator extends AbstractEJBValidator {
 				logger.write(Level.FINER, entry);
 			}
 		} finally {
-			if (ValidationRuleUtility.helperMap != null) {
-				ValidationRuleUtility.helperMap.clear();
-				ValidationRuleUtility.helperMap = null;
-			}
+
 		}
+	}	
+	
+	
+	public void validate(IValidationContext helper, IReporter reporter) throws ValidationException {
+		commonValidate(helper, reporter);
+	}
+	/*
+	 * @see IValidator#validate(IValidationContext, IReporter, IFileDelta[])
+	 */
+	public IStatus validateInJob(IValidationContext helper, IReporter reporter) throws ValidationException {
+		commonValidate(helper, reporter);
+		return status;
 	}
 	
 	public boolean isFullValidate(IEJBValidationContext vc) {
@@ -139,9 +148,6 @@ public class EJBValidator extends AbstractEJBValidator {
 			catch(ValidationException e) {
 				throw e;
 			}
-			catch(MessageLimitException e) {
-				throw e;
-			}
 			catch(Throwable exc) {
 				addInternalErrorMessage(getValidationContext(), exc);
 			}
@@ -337,9 +343,6 @@ public class EJBValidator extends AbstractEJBValidator {
 						catch(ValidationCancelledException e) {
 							throw e;
 						}
-						catch(MessageLimitException e) {
-							throw e;
-						}
 						catch(Throwable exc) {
 							addInternalErrorMessage(getValidationContext(), exc);
 						}
@@ -369,6 +372,7 @@ public class EJBValidator extends AbstractEJBValidator {
 	public void fullValidate(IEJBValidationContext vc) throws ValidationException {
 		removeOldMessages(vc,null); // null == no IFileDelta, null = don't track targets
 		
+
 		EJBJar ejbJar = (EJBJar)vc.loadModel(EJBValidatorModelEnum.EJB_MODEL);
 		if(ejbJar == null) {
 			// Log, add "Cannot validate" to task list, and return.
@@ -506,5 +510,23 @@ public class EJBValidator extends AbstractEJBValidator {
         EJBValidationRuleFactory.getFactory().release(rule);
         
     }
+
+	public ISchedulingRule getSchedulingRule(IValidationContext helper) {
+		// TODO Auto-generated method stub
+		return null;
+	}
 	
+	public void cleanup(IReporter reporter){
+		if( reporter instanceof WorkbenchReporter ){
+			WorkbenchReporter wbReporter = (WorkbenchReporter)reporter;
+			IProject project = wbReporter.getProject();
+			HashMap helperMap = ValidationRuleUtility.getHelperMap(project);
+			helperMap.clear();
+			ValidationRuleUtility.projectHelperMap.remove( helperMap );
+			helperMap = null;
+		}
+		// clear the map when the ejb validator is done see bug 187286
+		EJBValidationRuleFactory.getFactory().clearRuleMap(reporter);
+		super.cleanup(reporter);
+	}
 }
