@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004 IBM Corporation and others.
+ * Copyright (c) 2004, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -48,6 +48,8 @@ import org.eclipse.jst.jsp.core.internal.Logger;
 import org.eclipse.jst.jsp.core.internal.contentmodel.TaglibController;
 import org.eclipse.jst.jsp.core.internal.contentmodel.tld.TLDCMDocumentManager;
 import org.eclipse.jst.jsp.core.internal.contentmodel.tld.provisional.JSP12TLDNames;
+import org.eclipse.jst.jsp.core.internal.contentmodel.tld.provisional.TLDElementDeclaration;
+import org.eclipse.jst.jsp.core.internal.provisional.JSP11Namespace;
 import org.eclipse.jst.jsp.core.internal.regions.DOMJSPRegionContexts;
 import org.eclipse.jst.jsp.core.internal.taglib.TaglibHelper;
 import org.eclipse.jst.jsp.core.internal.taglib.TaglibHelperManager;
@@ -118,12 +120,12 @@ public class JSPTranslator {
 				" javax.servlet.http.HttpServletResponse response)" + ENDL + //$NON-NLS-1$
 				"\t\tthrows java.io.IOException, javax.servlet.ServletException {" + ENDL + //$NON-NLS-1$
 				"javax.servlet.jsp.PageContext pageContext = null;" + ENDL + //$NON-NLS-1$
-				"javax.servlet.http.HttpSession session = null;" + ENDL + //$NON-NLS-1$
 				"javax.servlet.ServletContext application = null;" + ENDL + //$NON-NLS-1$
 				"javax.servlet.ServletConfig config = null;" + ENDL + //$NON-NLS-1$ 
 				"javax.servlet.jsp.JspWriter out = null;" + ENDL + //$NON-NLS-1$
 				"Object page = null;" + ENDL; //$NON-NLS-1$
 
+	private String fSessionVariableDeclaration = "javax.servlet.http.HttpSession session = null;" + ENDL; //$NON-NLS-1$
 	private String fFooter = "}}"; //$NON-NLS-1$
 	private String fException = "Throwable exception = null;"; //$NON-NLS-1$
 	public static final String EXPRESSION_PREFIX = "out.print(\"\"+"; //$NON-NLS-1$
@@ -141,6 +143,7 @@ public class JSPTranslator {
 	private int fCursorPosition = -1;
 	/** some page directive attributes */
 	private boolean fIsErrorPage, fCursorInExpression = false;
+	private boolean fIsInASession = true;
 
 	/** user java code in body of the service method */
 	private StringBuffer fUserCode = new StringBuffer();
@@ -433,6 +436,7 @@ public class JSPTranslator {
 		fCursorPosition = -1;
 
 		fIsErrorPage = fCursorInExpression = false;
+		fIsInASession = true;
 
 		fUserCode = new StringBuffer();
 		fUserDeclarations = new StringBuffer();
@@ -534,6 +538,11 @@ public class JSPTranslator {
 			fResult.append(fException);
 			javaOffset += fException.length();
 		}
+		// session participant
+		if(fIsInASession) {
+			fResult.append(fSessionVariableDeclaration);
+			javaOffset += fSessionVariableDeclaration.length();
+		}
 
 
 		fResult.append(fTryCatchStart);
@@ -721,15 +730,18 @@ public class JSPTranslator {
 	public final String getJspText() {
 		return fJspTextBuffer.toString();
 	}
-
+	
 	protected void addTaglibVariables(String tagToAdd) {
+		addTaglibVariables(tagToAdd, getCurrentNode());
+	}
+
+	protected void addTaglibVariables(String tagToAdd, IStructuredDocumentRegion customTag) {
 		IFile f = getFile();
 
 		if (f == null || !f.exists())
 			return;
 
 		TaglibHelper helper = TaglibHelperManager.getInstance().getTaglibHelper(f);
-		IStructuredDocumentRegion customTag = getCurrentNode();
 		/*
 		 * Variables can declare as available when NESTED, AT_BEGIN, or
 		 * AT_END. For AT_END variables, store the entire list of variables in
@@ -1139,7 +1151,15 @@ public class JSPTranslator {
 							if (st.hasMoreTokens()) {
 								String directiveName = st.nextToken();
 								if (directiveName.equals("taglib")) { //$NON-NLS-1$
-									handleTaglib();
+									while (r != null && regions.hasNext() && !r.getType().equals(DOMRegionContext.XML_TAG_ATTRIBUTE_NAME)) {
+										r = (ITextRegion) regions.next();
+										if(container.getText(r).equals(JSP11Namespace.ATTR_NAME_PREFIX)) {
+											String prefix = getAttributeValue(r, regions);
+											if (prefix != null) {
+												handleTaglib(prefix);
+											}
+										}
+									}
 									return;
 								}
 								else if (directiveName.equals("include")) { //$NON-NLS-1$
@@ -1164,9 +1184,9 @@ public class JSPTranslator {
 									// see:
 									// https://w3.opensource.ibm.com/bugzilla/show_bug.cgi?id=3035
 									// setCurrentNode(getCurrentNode().getNext());
-									if (getCurrentNode() != null) {
+									if (container != null) {
 										// 'regions' contain the attrs
-										translatePageDirectiveAttributes(regions);
+										translatePageDirectiveAttributes(container, regions);
 									}
 								}
 							}
@@ -1361,7 +1381,7 @@ public class JSPTranslator {
 	 */
 	protected void translateJSPNode(ITextRegion region, Iterator regions, String type, int JSPType) {
 		if (type == DOMJSPRegionContexts.JSP_DIRECTIVE_OPEN && regions != null) {
-			translateDirective(regions);
+			translateDirective(fCurrentNode, regions);
 		}
 		else {
 			ITextRegionCollection contentRegion = null;
@@ -1584,7 +1604,7 @@ public class JSPTranslator {
 	/**
 	 * /* <%@ %> /* need to pass in the directive tag region
 	 */
-	protected void translateDirective(Iterator regions) {
+	protected void translateDirective(ITextRegionCollection container, Iterator regions) {
 		ITextRegion r = null;
 		String regionText, attrValue = ""; //$NON-NLS-1$
 		while (regions.hasNext() && (r = (ITextRegion) regions.next()) != null && r.getType() == DOMJSPRegionContexts.JSP_DIRECTIVE_NAME) { // could
@@ -1614,7 +1634,7 @@ public class JSPTranslator {
 					handleIncludeFile(fileLocation);
 			}
 			else if (regionText.indexOf("page") > -1) { //$NON-NLS-1$
-				translatePageDirectiveAttributes(regions);
+				translatePageDirectiveAttributes(container, regions);
 			}
 		}
 	}
@@ -1626,6 +1646,8 @@ public class JSPTranslator {
 	 * markers that came from <@taglib> directives, (not <@include>), since
 	 * include file taglibs are handled on the fly when they are encountered. *
 	 * @param regions
+	 * 
+	 * @deprecated - does not properly handle prefixes
 	 */
 	protected void handleTaglib() {
 		// get/create TLDCMDocument
@@ -1658,6 +1680,29 @@ public class JSPTranslator {
 		}
 	}
 
+	/*
+	 * This method should ideally only be called once per run through
+	 * JSPTranslator This is intended for use by inner helper classes that
+	 * need to add block markers to their own parsers. This method only adds
+	 * markers that came from <@taglib> directives, (not <@include>), since
+	 * include file taglibs are handled on the fly when they are encountered. *
+	 * @param regions
+	 */
+	private void handleTaglib(String prefix) {
+		// get/create TLDCMDocument
+		TLDCMDocumentManager mgr = TaglibController.getTLDCMDocumentManager(fStructuredDocument);
+		if (mgr != null) {
+			// get trackers for the CMDocuments enabled at this offset
+			List trackers = mgr.getCMDocumentTrackers(getCurrentNode().getEnd());
+			Iterator it = trackers.iterator();
+			CMDocumentTracker tracker = null;
+			while (it.hasNext()) {
+				tracker = (CMDocumentTracker) it.next();
+				addBlockMarkers(prefix + ":", tracker.getDocument());
+			}
+		}
+	}
+
 	/*
 	 * adds block markers to JSPTranslator's block marker list for all
 	 * elements in doc @param doc
@@ -1673,6 +1718,24 @@ public class JSPTranslator {
 		}
 	}
 
+	/*
+	 * adds block markers to JSPTranslator's block marker list for all
+	 * elements in doc @param doc
+	 */
+	protected void addBlockMarkers(String prefix, CMDocument doc) {
+		if (doc.getElements().getLength() > 0) {
+			Iterator elements = doc.getElements().iterator();
+			CMNode node = null;
+			while (elements.hasNext()) {
+				node = (CMNode) elements.next();
+				if (node instanceof TLDElementDeclaration && ((TLDElementDeclaration) node).getBodycontent().equals(JSP12TLDNames.CONTENT_TAGDEPENDENT))
+					getBlockMarkers().add(new BlockMarker(prefix + node.getNodeName(), null, DOMRegionContext.BLOCK_TEXT, true));
+				else
+					getBlockMarkers().add(new BlockMarker(prefix + node.getNodeName(), null, DOMJSPRegionContexts.JSP_CONTENT, true));
+			}
+		}
+	}
+
 	/**
 	 * If r is an attribute name region, this method will safely return the
 	 * value for that attribute.
@@ -1695,7 +1758,7 @@ public class JSPTranslator {
 	/**
 	 * takes an emnumeration of the attributes of a directive tag
 	 */
-	protected void translatePageDirectiveAttributes(Iterator regions) {
+	protected void translatePageDirectiveAttributes(ITextRegionCollection container, Iterator regions) {
 		ITextRegion r = null;
 		String attrName, attrValue;
 		// iterate all attributes
@@ -1703,12 +1766,12 @@ public class JSPTranslator {
 			attrName = attrValue = null;
 			if (r.getType().equals(DOMRegionContext.XML_TAG_ATTRIBUTE_NAME)) {
 
-				attrName = getCurrentNode().getText(r).trim();
+				attrName = container.getText(r).trim();
 				if (attrName.length() > 0) {
 					if (regions.hasNext() && (r = (ITextRegion) regions.next()) != null && r.getType() == DOMRegionContext.XML_TAG_ATTRIBUTE_EQUALS) {
 						if (regions.hasNext() && (r = (ITextRegion) regions.next()) != null && r.getType() == DOMRegionContext.XML_TAG_ATTRIBUTE_VALUE) {
 
-							attrValue = StringUtils.strip(getCurrentNode().getText(r));
+							attrValue = StringUtils.strip(container.getText(r));
 						}
 						// has equals, but no value?
 					}
@@ -1734,7 +1797,7 @@ public class JSPTranslator {
 		}
 		else if (attrName.equals("session")) //$NON-NLS-1$
 		{
-			// fSession = ("true".equalsIgnoreCase(attrValue)); //$NON-NLS-1$
+			fIsInASession = "true".equalsIgnoreCase(attrValue); //$NON-NLS-1$
 		}
 		else if (attrName.equals("buffer")) //$NON-NLS-1$
 		{
@@ -1750,7 +1813,7 @@ public class JSPTranslator {
 		}
 		else if (attrName.equals("isErrorPage")) //$NON-NLS-1$
 		{
-			fIsErrorPage = Boolean.valueOf(attrValue).booleanValue();
+			fIsErrorPage = "true".equalsIgnoreCase(attrValue); //$NON-NLS-1$
 		}
 	}
 
