@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005,2006 IBM Corporation and others.
+ * Copyright (c) 2005, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -552,6 +552,23 @@ class ProjectDescription {
 				projectsProcessed.add(buildpathProjects[i]);
 				ProjectDescription description = TaglibIndex._instance.createDescription(buildpathProjects[i]);
 				description.addBuildPathReferences(references, projectsProcessed, true);
+				
+				/*
+				 * 183756 - JSP Validation Cannot Find Tag Library Descriptor
+				 * in Referenced Projects
+				 * 
+				 * Add any TLD records having URI values from projects on the build path
+				 */
+				Map[] rootReferences = (Map[]) description.fImplicitReferences.values().toArray(new Map[description.fImplicitReferences.size()]);
+				for (int j = 0; j < rootReferences.length; j++) {
+					Iterator implicitRecords = rootReferences[j].values().iterator();
+					while (implicitRecords.hasNext()) {
+						ITaglibRecord record = (ITaglibRecord) implicitRecords.next();
+						if (record.getRecordType() == ITaglibRecord.TLD && ((ITLDRecord) record).getURI() != null) {
+							references.put(((ITLDRecord) record).getURI(), record);
+						}
+					}
+				}
 			}
 		}
 	}
@@ -1108,9 +1125,9 @@ class ProjectDescription {
 				break;
 			case IClasspathEntry.CPE_LIBRARY : {
 				/*
-				 * Ignore libs in required projects that are not exported
+				 * Track all libs now since required projects are consulted
+				 * for their build path jars separately
 				 */
-				if (fClasspathProjects.size() < 2 || entry.isExported()) {
 					IPath libPath = entry.getPath();
 					if (!fClasspathJars.containsKey(libPath.toString())) {
 						if (libPath.toFile().exists()) {
@@ -1129,7 +1146,6 @@ class ProjectDescription {
 							}
 						}
 					}
-				}
 			}
 				break;
 			case IClasspathEntry.CPE_PROJECT : {
@@ -1145,7 +1161,25 @@ class ProjectDescription {
 				break;
 			case IClasspathEntry.CPE_SOURCE :
 				break;
-			case IClasspathEntry.CPE_VARIABLE :
+			case IClasspathEntry.CPE_VARIABLE : {
+				IPath libPath = JavaCore.getResolvedVariablePath(entry.getPath());
+				if (libPath != null) {
+					File file = new File(libPath.toOSString());
+
+					// file in filesystem
+					if (file.exists() && !file.isDirectory()) {
+						updateClasspathLibrary(libPath.toString(), ITaglibRecordEvent.ADDED, entry.isExported());
+					}
+					else {
+						// workspace file
+						IFile jarFile = ResourcesPlugin.getWorkspace().getRoot().getFile(libPath);
+						if (jarFile.isAccessible() && jarFile.getType() == IResource.FILE && jarFile.getLocation() != null) {
+							String jarPathString = jarFile.getLocation().toString();
+							updateClasspathLibrary(jarPathString, ITaglibRecordEvent.ADDED, entry.isExported());
+						}
+					}
+				}
+			}
 				break;
 		}
 	}
