@@ -25,11 +25,12 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IExecutableExtension;
 import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.wst.rdb.internal.core.rte.DDLGenerator;
+import org.eclipse.wst.rdb.internal.core.rte.DDLGeneratorWithCallback;
 import org.eclipse.wst.rdb.internal.core.rte.EngineeringOption;
 import org.eclipse.wst.rdb.internal.core.rte.EngineeringOptionCategory;
 import org.eclipse.wst.rdb.internal.core.rte.EngineeringOptionCategoryID;
 import org.eclipse.wst.rdb.internal.core.rte.EngineeringOptionID;
+import org.eclipse.wst.rdb.internal.core.rte.IEngineeringCallBack;
 import org.eclipse.wst.rdb.internal.core.rte.fe.GenericDdlGenerationOptions;
 import org.eclipse.wst.rdb.internal.models.sql.constraints.CheckConstraint;
 import org.eclipse.wst.rdb.internal.models.sql.constraints.ForeignKey;
@@ -48,21 +49,43 @@ import org.eclipse.wst.rdb.internal.models.sql.tables.Table;
 import org.eclipse.wst.rdb.internal.models.sql.tables.Trigger;
 import org.eclipse.wst.rdb.internal.models.sql.tables.ViewTable;
 
-public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtension {
+public final class DerbyDdlGenerator implements DDLGeneratorWithCallback ,IExecutableExtension {
 	
 	public void setInitializationData(IConfigurationElement config, String propertyName, Object data) throws CoreException {
 		this.product = config.getAttribute("product"); //$NON-NLS-1$
 		this.version = config.getAttribute("version"); //$NON-NLS-1$
+		this.builder = new DerbyDdlBuilder();
 	}
 	
+	public String[] generateDDL(SQLObject[] elements, IProgressMonitor progressMonitor){
+		return this.generateDDL(elements, progressMonitor, null);
+	}
+
+    public String[] createSQLObjects(SQLObject[] elements, boolean quoteIdentifiers, boolean qualifyNames, IProgressMonitor progressMonitor){
+    	return this.createSQLObjects(elements, quoteIdentifiers, qualifyNames, progressMonitor,null);
+    }
+
+    public String[] dropSQLObjects(SQLObject[] elements, boolean quoteIdentifiers, boolean qualifyNames, IProgressMonitor progressMonitor){
+    	return this.dropSQLObjects(elements, quoteIdentifiers, qualifyNames, progressMonitor,null);
+    }
+
+
 	
-    public String[] generateDDL(SQLObject[] elements, IProgressMonitor progressMonitor) {
+	//@d00058820gs: use new structure/methods
+    public String[] generateDDL(SQLObject[] elements, IProgressMonitor progressMonitor,IEngineeringCallBack callback) {
+    	this.builder.setEngineeringCallBack(callback);
+    	String[] statements = new String[0];
+    	
     	EngineeringOption[] options = this.getSelectedOptions(elements);
-    	String[] statements = this.createSQLObjects(elements, EngineeringOptionID.generateQuotedIdentifiers(options),
-    	                      EngineeringOptionID.generateFullyQualifiedNames(options), progressMonitor);
-    	if(EngineeringOptionID.generateDropStatement(options)) {
-    		String[] drop = this.dropSQLObjects(elements, EngineeringOptionID.generateQuotedIdentifiers(options),
-    	                    EngineeringOptionID.generateFullyQualifiedNames(options), progressMonitor);    	
+        
+    	if (EngineeringOptionID.generateCreateStatement(options)) {
+    		statements = this.createSQLObjects(elements, EngineeringOptionID.generateQuotedIdentifiers(options),
+                    EngineeringOptionID.generateFullyQualifiedNames(options), progressMonitor);
+    	}
+        
+        if(EngineeringOptionID.generateDropStatement(options)) {
+        	String[] drop = this.dropSQLObjects(elements, EngineeringOptionID.generateQuotedIdentifiers(options),
+                    EngineeringOptionID.generateFullyQualifiedNames(options), progressMonitor);
             String[] temp = statements;
             statements = new String[temp.length + drop.length];
             for(int i=0; i<drop.length; ++i) {
@@ -75,20 +98,23 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
         return statements;
     }
 
-    public String[] createSQLObjects(SQLObject[] elements, boolean quoteIdentifiers, boolean qualifyNames, IProgressMonitor progressMonitor) {
+    public String[] createSQLObjects(SQLObject[] elements, boolean quoteIdentifiers, boolean qualifyNames, IProgressMonitor progressMonitor,IEngineeringCallBack callback) {
+    	this.builder.setEngineeringCallBack(callback);
     	return this.createStatements(elements,quoteIdentifiers,qualifyNames,progressMonitor,100);
     }
     	
-    public String[] dropSQLObjects(SQLObject[] elements, boolean quoteIdentifiers, boolean qualifyNames, IProgressMonitor progressMonitor) {
+    public String[] dropSQLObjects(SQLObject[] elements, boolean quoteIdentifiers, boolean qualifyNames, IProgressMonitor progressMonitor,IEngineeringCallBack callback) {
+    	this.builder.setEngineeringCallBack(callback);
     	return this.dropStatements(elements,quoteIdentifiers,qualifyNames,progressMonitor,100);
     }
     
+    //@d00058820gs: use new structure/methods
     private String[] createStatements(SQLObject[] elements, boolean quoteIdentifiers, boolean qualifyNames, IProgressMonitor progressMonitor, int task) {
         DerbyDdlScript script = new DerbyDdlScript();
         if(this.builder == null) {
             this.builder = new DerbyDdlBuilder();
         }
-
+        
         Set set = null;
         if (optionDependency != null)
             set = optionDependency.getElementsToUse();
@@ -100,7 +126,7 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
             it = GenericDdlGenerationOptions.getAllContainedDisplayableElementSet(elements).iterator();
         
         EngineeringOption[] options = this.getSelectedOptions(elements);
-
+        
         while(it.hasNext()) {
             Object o = it.next();
             if(o instanceof BaseTable) {
@@ -129,17 +155,17 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
                 if(statement != null) script.addCreateTriggerStatement(statement);
             }
             else if(o instanceof CheckConstraint) {
-            	if (!EngineeringOptionID.generateCKConstraints(options)) continue;
+                if (!EngineeringOptionID.generateCKConstraints(options)) continue;
             	String statement = builder.addCheckConstraint((CheckConstraint) o, quoteIdentifiers, qualifyNames);
                 if(statement != null) script.addAlterTableAddConstraintStatement(statement);
             }
             else if(o instanceof UniqueConstraint) {
-            	if(!EngineeringOptionID.generatePKConstraints(options)) continue;
+            	if(!EngineeringOptionID.generatePKConstraints(options) || builder.isImplicitConstraint((UniqueConstraint)o)) continue;
                 String statement = builder.addUniqueConstraint((UniqueConstraint) o, quoteIdentifiers, qualifyNames);
                 if(statement != null) script.addAlterTableAddConstraintStatement(statement);
             }
             else if(o instanceof ForeignKey) {
-            	if(!EngineeringOptionID.generateFKConstraints(options)) continue;
+            	if(!EngineeringOptionID.generateFKConstraints(options) || builder.isImplicitConstraint((ForeignKey)o)) continue;
                 String statement = builder.addForeignKey((ForeignKey) o, quoteIdentifiers, qualifyNames);
                 if(statement != null) script.addAlterTableAddForeignKeyStatement(statement);
             }
@@ -152,6 +178,7 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
         return script.getStatements();
     }
     
+    //@d00058820gs: use new structure/methods
     private String[] dropStatements(SQLObject[] elements, boolean quoteIdentifiers, boolean qualifyNames, IProgressMonitor progressMonitor, int task) {
         DerbyDdlScript script = new DerbyDdlScript();
         if(this.builder == null) {
@@ -159,7 +186,6 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
         }
         Iterator it = GenericDdlGenerationOptions.getAllContainedDisplayableElementSet(elements).iterator();
         EngineeringOption[] options = this.getSelectedOptions(elements);
-
         while(it.hasNext()) {
             Object o = it.next();
             if(o instanceof  BaseTable) {
@@ -188,17 +214,17 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
                 if(statement != null) script.addDropTriggerStatement(statement);
             }
             else if(o instanceof CheckConstraint) {
-            	if (!EngineeringOptionID.generateCKConstraints(options)) continue;
+                if (!EngineeringOptionID.generateCKConstraints(options)) continue;
                 String statement = builder.dropTableConstraint((CheckConstraint) o, quoteIdentifiers, qualifyNames);
                 if(statement != null) script.addAlterTableDropConstraintStatement(statement);
             }
             else if(o instanceof UniqueConstraint) {
-            	if(!EngineeringOptionID.generatePKConstraints(options)) continue;
+            	if(!EngineeringOptionID.generatePKConstraints(options) || builder.isImplicitConstraint((UniqueConstraint)o)) continue;
                 String statement = builder.dropTableConstraint((UniqueConstraint) o, quoteIdentifiers, qualifyNames);
                 if(statement != null) script.addAlterTableDropConstraintStatement(statement);
             }
             else if(o instanceof ForeignKey) {
-            	if(!EngineeringOptionID.generateFKConstraints(options)) continue;
+            	if(!EngineeringOptionID.generateFKConstraints(options) || builder.isImplicitConstraint((ForeignKey)o)) continue;
                 String statement = builder.dropTableConstraint((ForeignKey) o, quoteIdentifiers, qualifyNames);
                 if(statement != null) script.addAlterTableDropForeignKeyStatement(statement);
             }
@@ -238,42 +264,55 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
         return this.options;
     }
     
-	public EngineeringOption[] getOptions(SQLObject[] elements) {
-	       return getOptions(elements, false);
-	}
-	     
-	public EngineeringOption[] getOptions(SQLObject[] elements, boolean autoDiscovery) {
-        EngineeringOptionCategory[] categories_new = this.getOptionCategories();
-        
-        EngineeringOptionCategory general_options =null;
-        EngineeringOptionCategory additional_element =null;
-        for (int i = 0; i < categories_new.length; i++) {
-          if (categories_new[i].getId().equals(EngineeringOptionCategoryID.GENERATE_OPTIONS)){
-              general_options = categories_new[i];
-          } else if (categories_new[i].getId().equals(EngineeringOptionCategoryID.GENERATE_ELEMENTS)){
-              additional_element = categories_new[i];
-          }
-        }           
+    //@bd00058820gs
+    public EngineeringOption[] getOptions(SQLObject[] elements) {
+        return getOptions(elements, false);
+    }
+    
+    public EngineeringOption[] getOptions(SQLObject[] elements, boolean autoDiscovery) {
+        //if(this.selectedObject  == null) {
+            //Look to see if the objects differ from the previous selection
+            //Performance wise might not be a good idea
+//            for (int i=0; i < selectedObject.length; i++)
+//            {
+//                selectedObject
+//            }
 
-        //Get the main object, the ones that do no have parents.
-        //Look for dependency   
-        int idx = 0, size = 0;
-        optionDependency = new OptionDependency(elements, autoDiscovery);
-        Set sOptions = optionDependency.getOptions();
+            EngineeringOptionCategory[] categories_new = this.getOptionCategories();
+            
+            EngineeringOptionCategory general_options =null;
+            EngineeringOptionCategory additional_element =null;
+            for (int i = 0; i < categories_new.length; i++) {
+              if (categories_new[i].getId().equals(EngineeringOptionCategoryID.GENERATE_OPTIONS)){
+                  general_options = categories_new[i];
+              } else if (categories_new[i].getId().equals(EngineeringOptionCategoryID.GENERATE_ELEMENTS)){
+                  additional_element = categories_new[i];
+              }
+            }           
 
-        this.options = new EngineeringOption[sOptions.size()];
-        int i = 0;
-        for (Iterator it=sOptions.iterator(); it.hasNext(); i++) {
-            this.options[i] = GenericDdlGenerationOptions.getEngineeringOption((String)it.next(), general_options, additional_element);
-            if (this.options[i] != null && this.options[i].getCategory().getId().equals(EngineeringOptionCategoryID.GENERATE_ELEMENTS)) {
-            	idx = i;
-            	size++;
+            //Get the main object, the ones that do no have parents.
+            //Look for dependency
+            int idx = 0, size = 0;
+            optionDependency = new OptionDependency(elements, autoDiscovery);
+            Set sOptions = optionDependency.getOptions();
+
+            this.options = new EngineeringOption[sOptions.size()];
+            int i = 0;
+            for (Iterator it=sOptions.iterator(); it.hasNext(); i++) {
+                this.options[i] = GenericDdlGenerationOptions.getEngineeringOption((String)it.next(), general_options, additional_element);
+                //@bd00061447gs
+                if (this.options[i] != null && this.options[i].getCategory().getId().equals(EngineeringOptionCategoryID.GENERATE_ELEMENTS)) {
+                	idx = i;
+                	size++;
+                }
             }
-        }
-        if (size == 1) {
-        	EngineeringOption option = this.options[idx];
-        	option.setBoolean(true);
-        }       
+            if (size == 1) {
+            	EngineeringOption option = this.options[idx];
+            	option.setBoolean(true);
+            }
+            //@ed00061447gs
+        //}
+        
         return this.options;
     }
     
@@ -283,50 +322,53 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
         Set s = new TreeSet();
         for(int i=0; i<elements.length; ++i) {
             Class key = null;
+
             if(elements[i] instanceof Database) {
                 key = Database.class;
             } else if(elements[i] instanceof Schema) {
-            	key = Schema.class;
-           	} else if (elements[i] instanceof BaseTable) {
-           		key = Table.class;
-           	} else if (elements[i] instanceof Index) {
-            	key = Index.class;
-           	} else if (elements[i] instanceof Procedure) {
-            	key = Procedure.class;
-           	} else if (elements[i] instanceof UserDefinedFunction) {
-            	key = UserDefinedFunction.class;
-           	} else if (elements[i] instanceof ViewTable) {
-            	key = ViewTable.class;
-           	} else if (elements[i] instanceof Trigger) {
-            	key = Trigger.class;
-           	} else if (elements[i] instanceof UserDefinedType) {
-            	key = UserDefinedType.class;
-           	} else if (elements[i] instanceof UniqueConstraint) {
-            	key = UniqueConstraint.class;
-           	} else if(elements[i] instanceof CheckConstraint) {
-            	key = CheckConstraint.class;
-           	} else if(elements[i] instanceof ForeignKey) {
-            	key = ForeignKey.class;
-           	}
+                key = Schema.class;
+            } else if (elements[i] instanceof BaseTable) {
+                 key = Table.class;
+            } else if (elements[i] instanceof Index) {
+                key = Index.class;
+            } else if (elements[i] instanceof Procedure) {
+                key = Procedure.class;
+            } else if (elements[i] instanceof UserDefinedFunction) {
+                key = UserDefinedFunction.class;
+            } else if (elements[i] instanceof ViewTable) {
+                key = ViewTable.class;
+            } else if (elements[i] instanceof Trigger) {
+                key = Trigger.class;
+            } else if (elements[i] instanceof UserDefinedType) {
+                key = UserDefinedType.class;
+            } else if (elements[i] instanceof UniqueConstraint) {
+                key = UniqueConstraint.class;
+            } else if(elements[i] instanceof CheckConstraint) {
+                key = CheckConstraint.class;
+            } else if(elements[i] instanceof ForeignKey) {
+               key = ForeignKey.class;
+            }
 
             try {
-            	int mask = sod.getMask(key).intValue();
-            	EngineeringOptionID.populateOptions(s, mask);
+                int mask = sod.getMask(key).intValue();
+                EngineeringOptionID.populateOptions(s, mask);
             } catch (Exception e) {
-            	System.err.println("Missing definition for: " + elements[i].getClass().toString());
-            	e.printStackTrace();
+//                System.err.println("Missing definition for: " + elements[i].getClass().toString());
+//                e.printStackTrace();
             }
         }
         return s;
     }
+    //@ed00058820gs
     
     public EngineeringOptionCategory[] getOptionCategories() {
         if(this.categories == null) {
-           this.categories = GenericDdlGenerationOptions.createDDLGenerationOptionCategories();
+            this.categories = GenericDdlGenerationOptions.createDDLGenerationOptionCategories();
         }
         return this.categories;
     }
-
+    
+    //@bd00058820gs
     public EngineeringOption[] getSelectedOptions() {
         if (options == null)
             this.getOptions();
@@ -341,14 +383,17 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
                 this.getOptions();
         return options;
     }
-    
+    //@ed00058820gs
+
     private String product;
     private String version;
     private EngineeringOption[] options = null;
     private EngineeringOptionCategory[] categories = null;
 	private DerbyDdlBuilder builder = null;
-	private OptionDependency optionDependency = null;
-	
+
+    //@bd00058820gs
+    private OptionDependency optionDependency = null;
+    
     public class OptionDependency {
         private SQLObject[] elements = null;
         private Set sOptions = new LinkedHashSet();
@@ -367,6 +412,9 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
               setOption(EngineeringOptionID.GENERATE_QUOTED_IDENTIFIER);
               setOption(EngineeringOptionID.GENERATE_DROP_STATEMENTS);
               setOption(EngineeringOptionID.GENERATE_CREATE_STATEMENTS);
+              //setOption(EngineeringOptionID.GENERATE_COMMENTS);
+              //setOption(EngineeringOptionID.GENERATE_IN_TABLESPACE_CLAUSE);
+              //setOption(EngineeringOptionID.GENERATE_USE_DOMAIN_IF_EXIST);
 
               Set additionalOptions = new TreeSet();
               if (autoDiscovery) {
@@ -393,12 +441,15 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
                         additionalOptions.add(EngineeringOptionID.GENERATE_CK_CONSTRAINTS);
                       } else if(o instanceof ForeignKey) {
                         additionalOptions.add(EngineeringOptionID.GENERATE_FK_CONSTRAINTS);
+    //                  } else if(o instanceof Schema) {
+    //                    additionalOptions.add(EngineeringOptionID.GENERATE_SCHEMA);
                       } else if(o instanceof Column) {
+                        //additionalOptions.add(EngineeringOptionID.GENERATE_TABLES);
                       }
                   }
                   for (it = additionalOptions.iterator(); it.hasNext(); )
                     setOption((String)it.next());
-              } else {
+              } else { //autoDiscovery == FALSE
                   sOptions.addAll(DerbyDdlGenerator.getAllContainedDisplayableElementSetDepedency(elements));
               }
         }
@@ -424,14 +475,14 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
     }
     
     public static class SingletonOptionDependency {
-		 
+
         private Map data = new HashMap();
         private static SingletonOptionDependency ref;
         
         private SingletonOptionDependency()
         {
         }
-		 
+
         public static SingletonOptionDependency getSingletonObject()
         {
             if (ref == null) {
@@ -510,5 +561,6 @@ public final class DerbyDdlGenerator implements DDLGenerator ,IExecutableExtensi
             return (Integer)data.get(key);
         }
     }
+    //@ed00058820gs
 }
 
