@@ -1,5 +1,5 @@
 /**********************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -14,6 +14,9 @@ import java.util.*;
 
 import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.*;
+import org.eclipse.core.runtime.jobs.ISchedulingRule;
+import org.eclipse.core.runtime.jobs.Job;
+import org.eclipse.core.runtime.jobs.MultiRule;
 import org.eclipse.debug.core.*;
 
 import org.eclipse.osgi.util.NLS;
@@ -71,10 +74,10 @@ public class Server extends Base implements IServer {
 
 	protected IRuntime runtime;
 	protected IFolder configuration;
-	
+
 	// the list of modules that are to be published to the server
 	protected List modules;
-	
+
 	// transient fields
 	protected transient String mode = ILaunchManager.RUN_MODE;
 	protected transient int serverState = STATE_UNKNOWN;
@@ -84,7 +87,7 @@ public class Server extends Base implements IServer {
 	protected transient Map moduleState = new HashMap();
 	protected transient Map modulePublishState = new HashMap();
 	protected transient Map moduleRestartState = new HashMap();
-	
+
 	protected transient IStatus serverStatus;
 	protected transient Map moduleStatus = new HashMap();
 
@@ -134,6 +137,70 @@ public class Server extends Base implements IServer {
 		}
 	}
 
+	public class ResourceChangeJob extends ChainedJob {
+		private IModule module;
+
+		public ResourceChangeJob(IModule module, IServer server) {
+			super(NLS.bind(Messages.jobUpdateServer, server.getName()), server);
+			this.module = module;
+			
+			if (module.getProject() == null)
+				setRule(new ServerSchedulingRule(server));
+			else {
+				ISchedulingRule[] rules = new ISchedulingRule[2];
+				IResourceRuleFactory ruleFactory = ResourcesPlugin.getWorkspace().getRuleFactory();
+				rules[0] = ruleFactory.createRule(module.getProject());
+				rules[1] = new ServerSchedulingRule(server);
+				setRule(MultiRule.combine(rules));
+			}
+		}
+
+		protected IModule getModule() {
+			return module;
+		}
+
+		protected IStatus run(IProgressMonitor monitor) {
+			final boolean[] changed = new boolean[1];
+			final List modules2 = new ArrayList();
+			
+			IModuleVisitor visitor = new IModuleVisitor() {
+				public boolean visit(IModule[] module2) {
+					modules2.add(module2);
+					
+					int size = module2.length;
+					IModule m = module2[size - 1];
+					if (m.getProject() == null)
+						return true;
+					
+					if (module.equals(m)) {
+						if (hasPublishedResourceDelta(module2)) {
+							changed[0] = true;
+							setModulePublishState(module2, IServer.PUBLISH_STATE_INCREMENTAL);
+						}
+					}
+					return true;
+				}
+			};
+			
+			visit(visitor, null);
+			
+			if (getServerPublishInfo().hasStructureChanged(modules2))
+				setServerPublishState(IServer.PUBLISH_STATE_INCREMENTAL);
+			
+			if (!changed[0])
+				//return;
+				return Status.OK_STATUS;
+			
+			if (getServerState() != IServer.STATE_STOPPED && behaviourDelegate != null)
+				behaviourDelegate.handleResourceChange();
+			
+			if (getServerState() == IServer.STATE_STARTED)
+				autoPublish();
+			
+			return Status.OK_STATUS;
+		}
+	};
+	
 	private static final Comparator PUBLISH_OPERATION_COMPARTOR = new Comparator() {
       public int compare(Object leftOp, Object rightOp) {
           PublishOperation left = (PublishOperation) leftOp;
@@ -178,10 +245,20 @@ public class Server extends Base implements IServer {
 		return false;
 	}
 	
-	protected void deleteFromMetadata() {
+    protected void deleteFromFile() throws CoreException {
+        super.deleteFromFile();
+        ResourceManager.getInstance().deregisterServer(this);
+    }
+
+    protected void deleteFromMetadata() {
 		ResourceManager.getInstance().removeServer(this);
 	}
-	
+
+	protected void saveToFile(IProgressMonitor monitor) throws CoreException {
+		super.saveToFile(monitor);
+		ResourceManager.getInstance().registerServer(this);
+	}
+
 	protected void saveToMetadata(IProgressMonitor monitor) {
 		super.saveToMetadata(monitor);
 		ResourceManager.getInstance().addServer(this);
@@ -235,6 +312,9 @@ public class Server extends Base implements IServer {
 					behaviourDelegate = ((ServerType) serverType).createServerBehaviourDelegate();
 					InternalInitializer.initializeServerBehaviourDelegate(behaviourDelegate, Server.this, monitor);
 					Trace.trace(Trace.PERFORMANCE, "Server.getBehaviourDelegate(): <" + (System.currentTimeMillis() - time) + "> " + getServerType().getId());
+					
+					if (getServerState() == IServer.STATE_STARTED)
+						autoPublish();
 				} catch (Throwable t) {
 					Trace.trace(Trace.SEVERE, "Could not create behaviour delegate " + toString(), t);
 				}
@@ -261,7 +341,7 @@ public class Server extends Base implements IServer {
 	public int getAutoPublishTime() {
 		return getAttribute(PROP_AUTO_PUBLISH_TIME, -1);
 	}
-	
+
 	public int getAutoPublishSetting() {
 		return getAttribute(PROP_AUTO_PUBLISH_SETTING, AUTO_PUBLISH_DEFAULT);
 	}
@@ -349,13 +429,21 @@ public class Server extends Base implements IServer {
 	 */
 	protected void fireRestartStateChangeEvent() {
 		Trace.trace(Trace.LISTENERS, "->- Firing server restart change event: " + getName() + " ->-");
-	
+		
 		if (notificationManager == null || notificationManager.hasListenerEntries())
 			return;
-	
+		
 		notificationManager.broadcastChange(
 			new ServerEvent(ServerEvent.SERVER_CHANGE | ServerEvent.RESTART_STATE_CHANGE, this, getServerState(), 
 				getServerPublishState(), getServerRestartState()));
+		
+		if (getServerState() == IServer.STATE_STARTED && getServerRestartState() && ServerPreferences.getInstance().isAutoRestarting()) {
+			String mode = getMode();
+			if (canRestart(mode).isOK()) {
+				RestartServerJob job = new RestartServerJob(this, mode);
+				job.schedule();
+			}
+		}
 	}
 
 	/**
@@ -425,6 +513,10 @@ public class Server extends Base implements IServer {
 	public void setModuleState(IModule[] module, int state) {
 		if (module == null)
 			throw new IllegalArgumentException("Module cannot be null");
+		int oldState = getModuleState(module);
+		if (oldState == state)
+			return;
+		
 		Integer in = new Integer(state);
 		moduleState.put(getKey(module), in);
 		fireModuleStateChangeEvent(module);
@@ -433,6 +525,10 @@ public class Server extends Base implements IServer {
 	public void setModulePublishState(IModule[] module, int state) {
 		if (module == null)
 			throw new IllegalArgumentException("Module cannot be null");
+		int oldState = getModulePublishState(module);
+		if (oldState == state)
+			return;
+		
 		Integer in = new Integer(state);
 		if (state == -1)
 			modulePublishState.remove(getKey(module));
@@ -443,53 +539,35 @@ public class Server extends Base implements IServer {
 	public void setModuleRestartState(IModule[] module, boolean r) {
 		if (module == null)
 			throw new IllegalArgumentException("Module cannot be null");
+		boolean oldState = getModuleRestartState(module);
+		if (oldState == r)
+			return;
+		
 		Boolean b = new Boolean(r);
 		moduleState.put(getKey(module), b);
 		fireModuleRestartChangeEvent(module);
 	}
 
-	protected void handleModuleProjectChange(final IModule module) {
+	protected void handleModuleProjectChange(IModule module) {
 		Trace.trace(Trace.FINEST, "> handleDeployableProjectChange() " + this + " " + module);
 		
-		class Helper {
-			boolean changed;
-		}
-		final Helper helper = new Helper();
-		
-		final List modules2 = new ArrayList();
-		
-		IModuleVisitor visitor = new IModuleVisitor() {
-			public boolean visit(IModule[] module2) {
-				modules2.add(module2);
-				
-				int size = module2.length;
-				IModule m = module2[size - 1];
-				if (m.getProject() == null)
-					return true;
-				
-				if (module.equals(m)) {
-					if (hasPublishedResourceDelta(module2)) {
-						helper.changed = true;
-						setModulePublishState(module2, IServer.PUBLISH_STATE_INCREMENTAL);
-					}
+		// check for duplicate jobs already waiting and don't create a new one
+		Job[] jobs = Platform.getJobManager().find(ServerPlugin.PLUGIN_ID);
+		if (jobs != null) {
+			int size = jobs.length;
+			for (int i = 0; i < size; i++) {
+				if (jobs[i] instanceof ResourceChangeJob) {
+					ResourceChangeJob rcj = (ResourceChangeJob) jobs[i];
+					if (rcj.getServer().equals(this) && rcj.getModule().equals(module) && rcj.getState() == Job.WAITING)
+						return;
 				}
-				return true;
 			}
-		};
+		}
 		
-		visit(visitor, null);
-		
-		if (getServerPublishInfo().hasStructureChanged(modules2))
-			setServerPublishState(IServer.PUBLISH_STATE_INCREMENTAL);
-		
-		if (!helper.changed)
-			return;
-		
-		if (getServerState() != IServer.STATE_STOPPED && behaviourDelegate != null)
-			behaviourDelegate.handleResourceChange();
-		
-		if (getServerState() == IServer.STATE_STARTED)
-			autoPublish();
+		ResourceChangeJob job = new ResourceChangeJob(module, this);
+		job.setSystem(true);
+		job.setPriority(Job.BUILD);
+		job.schedule();
 		
 		Trace.trace(Trace.FINEST, "< handleDeployableProjectChange()");
 	}
@@ -706,22 +784,19 @@ public class Server extends Base implements IServer {
 		if (getServerPublishState() != PUBLISH_STATE_NONE)
 			return true;
 		
-		class Temp {
-			boolean publish;
-		}
-		final Temp temp = new Temp();
+		final boolean[] publish = new boolean[1];
 		
 		visit(new IModuleVisitor() {
 			public boolean visit(IModule[] module) {
 				if (getModulePublishState(module) != PUBLISH_STATE_NONE) {
-					temp.publish = true;
+					publish[0] = true;
 					return false;
 				}
 				return true;
 			}
 		}, null);
 		
-		return temp.publish;
+		return publish[0];
 	}
 
 	/**
@@ -738,22 +813,19 @@ public class Server extends Base implements IServer {
 		if (getServerRestartState())
 			return true;
 		
-		class Temp {
-			boolean publish;
-		}
-		final Temp temp = new Temp();
+		final boolean[] publish = new boolean[1];
 		
 		visit(new IModuleVisitor() {
 			public boolean visit(IModule[] module) {
 				if (getModuleRestartState(module)) {
-					temp.publish = true;
+					publish[0] = true;
 					return false;
 				}
 				return true;
 			}
 		}, null);
 		
-		return temp.publish;
+		return publish[0];
 	}
 
 	public ServerPublishInfo getServerPublishInfo() {
@@ -813,7 +885,7 @@ public class Server extends Base implements IServer {
 				}
 			}, monitor);
 			
-			getServerPublishInfo().removeDeletedModulePublishInfo(modules2);
+			getServerPublishInfo().removeDeletedModulePublishInfo(this, modules2);
 			getServerPublishInfo().clearCache();
 			getServerPublishInfo().save();
 			
@@ -881,7 +953,7 @@ public class Server extends Base implements IServer {
 		
 		return (PublishOperation[]) tasks.toArray(new PublishOperation[tasks.size()]);
 	}
-	
+
 	/**
 	 * Returns all publish tasks that have been targetted to this server type.
 	 * The tasks will not be initialized with a task model. 
@@ -1095,10 +1167,34 @@ public class Server extends Base implements IServer {
 				try {
 					String serverId = launchConfigs[i].getAttribute(SERVER_ID, (String) null);
 					if (getId().equals(serverId)) {
-						ILaunchConfigurationWorkingCopy wc = launchConfigs[i].getWorkingCopy();
+						final ILaunchConfigurationWorkingCopy wc = launchConfigs[i].getWorkingCopy();
 						setupLaunchConfiguration(wc, monitor);
-						if (wc.isDirty())
-							return wc.doSave();
+						if (wc.isDirty()) {
+							final ILaunchConfiguration[] lc = new ILaunchConfiguration[1];
+							Job job = new Job("Saving launch configuration") {
+								protected IStatus run(IProgressMonitor monitor) {
+									try {
+										lc[0] = wc.doSave();
+									} catch (CoreException ce) {
+										Trace.trace(Trace.SEVERE, "Error configuring launch", ce);
+									}
+									return Status.OK_STATUS;
+								}
+							};
+							job.setSystem(true);
+							job.schedule();
+							try {
+								job.join();
+							} catch (Exception e) {
+								Trace.trace(Trace.SEVERE, "Error configuring launch", e);
+							}
+							if (job.getState() != Job.NONE) {
+								job.cancel();
+								lc[0] = wc.doSave();
+							}
+							
+							return lc[0];
+						}
 						return launchConfigs[i];
 					}
 				} catch (CoreException e) {
@@ -1135,6 +1231,9 @@ public class Server extends Base implements IServer {
 	public void start(String mode2, IProgressMonitor monitor) throws CoreException {
 		Trace.trace(Trace.FINEST, "Starting server: " + toString() + ", launchMode: " + mode2);
 	
+		// make sure that the delegate is loaded and the server state is correct
+		loadAdapter(ServerBehaviourDelegate.class, monitor);
+		
 		try {
 			ILaunchConfiguration launchConfig = getLaunchConfiguration(true, monitor);
 			ILaunch launch = launchConfig.launch(mode2, monitor); // , true); - causes workspace lock
@@ -1145,6 +1244,16 @@ public class Server extends Base implements IServer {
 		}
 	}
 
+	/**
+	 * Clean up any metadata associated with the server, typically in preparation for
+	 * deletion.
+	 */
+	protected void deleteMetadata() {
+		deleteLaunchConfigurations();
+		ServerPlugin.getInstance().removeTempDirectory(getId());
+		PublishInfo.getInstance().removeServerPublishInfo(this);
+	}
+
 	/**
 	 * Clean up any old launch configurations with the current server's id.
 	 */
@@ -1290,7 +1399,10 @@ public class Server extends Base implements IServer {
 	public IStatus canStop() {
 		if (getServerState() == STATE_STOPPED)
 			return new Status(IStatus.ERROR, ServerPlugin.PLUGIN_ID, 0, Messages.errorStopAlreadyStopped, null);
-
+		
+		if (getServerType() != null && !getServerType().supportsLaunchMode(getMode()))
+			return new Status(IStatus.ERROR, ServerPlugin.PLUGIN_ID, 0, Messages.errorLaunchMode, null);
+		
 		return Status.OK_STATUS;
 	}
 
@@ -1316,7 +1428,10 @@ public class Server extends Base implements IServer {
 	public void start(String mode2, IOperationListener listener2) {
 		Trace.trace(Trace.FINEST, "synchronousStart 1");
 		final Object mutex = new Object();
-	
+		
+		// make sure that the delegate is loaded and the server state is correct
+		loadAdapter(ServerBehaviourDelegate.class, null);
+		
 		// add listener to the server
 		IServerListener listener = new IServerListener() {
 			public void serverChanged(ServerEvent event) {
@@ -1340,47 +1455,57 @@ public class Server extends Base implements IServer {
 		};
 		addServerListener(listener);
 		
-		final int serverTimeout = ((ServerType) getServerType()).getStartTimeout();
 		class Timer {
 			boolean timeout;
 			boolean alreadyDone;
 		}
 		final Timer timer = new Timer();
-			
-		Thread thread = new Thread() {
-			public void run() {
-				try {
-					Thread.sleep(serverTimeout * 1000);
-					if (!timer.alreadyDone) {
-						timer.timeout = true;
-						// notify waiter
-						synchronized (mutex) {
-							Trace.trace(Trace.FINEST, "synchronousStart notify timeout");
-							mutex.notifyAll();
+		
+		final int serverTimeout = ((ServerType) getServerType()).getStartTimeout();
+		if (serverTimeout > 0) {
+			Thread thread = new Thread("Server start timeout") {
+				public void run() {
+					try {
+						Thread.sleep(serverTimeout);
+						if (!timer.alreadyDone) {
+							// notify waiter
+							synchronized (mutex) {
+								Trace.trace(Trace.FINEST, "start notify timeout");
+								if (!timer.alreadyDone)
+									timer.timeout = true;
+								mutex.notifyAll();
+							}
 						}
+					} catch (Exception e) {
+						Trace.trace(Trace.SEVERE, "Error notifying server start timeout", e);
 					}
-				} catch (Exception e) {
-					Trace.trace(Trace.SEVERE, "Error notifying server start timeout", e);
 				}
-			}
-		};
-		thread.setDaemon(true);
-		thread.start();
+			};
+			thread.setDaemon(true);
+			thread.start();
+		}
 		
 		Trace.trace(Trace.FINEST, "synchronousStart 2");
-	
+		
 		// start the server
+		IProgressMonitor monitor = new NullProgressMonitor();
 		try {
-			start(mode2, (IProgressMonitor)null);
+			start(mode2, monitor);
 		} catch (CoreException e) {
 			removeServerListener(listener);
 			timer.alreadyDone = true;
 			listener2.done(e.getStatus());
 			return;
 		}
-	
+		if (monitor.isCanceled()) {
+			removeServerListener(listener);
+			timer.alreadyDone = true;
+			listener2.done(Status.CANCEL_STATUS);
+			return;
+		}
+		
 		Trace.trace(Trace.FINEST, "synchronousStart 3");
-	
+		
 		// wait for it! wait for it! ...
 		synchronized (mutex) {
 			try {
@@ -1389,29 +1514,33 @@ public class Server extends Base implements IServer {
 			} catch (Exception e) {
 				Trace.trace(Trace.SEVERE, "Error waiting for server start", e);
 			}
+			timer.alreadyDone = true;
 		}
 		removeServerListener(listener);
-		timer.alreadyDone = true;
 		
 		if (timer.timeout) {
 			listener2.done(new Status(IStatus.ERROR, ServerPlugin.PLUGIN_ID, 0, NLS.bind(Messages.errorStartTimeout, new String[] { getName(), (serverTimeout / 1000) + "" }), null));
+			stop(false);
 			return;
 		}
-		timer.alreadyDone = true;
 		
 		if (getServerState() == IServer.STATE_STOPPED) {
 			listener2.done(new Status(IStatus.ERROR, ServerPlugin.PLUGIN_ID, 0, NLS.bind(Messages.errorStartFailed, getName()), null));
 			return;
 		}
-	
+		
 		Trace.trace(Trace.FINEST, "synchronousStart 4");
 		listener2.done(Status.OK_STATUS);
 	}
 
 	public void synchronousStart(String mode2, IProgressMonitor monitor) throws CoreException {
 		Trace.trace(Trace.FINEST, "synchronousStart 1");
-		final Object mutex = new Object();
-	
+		
+		// make sure that the delegate is loaded and the server state is correct
+		loadAdapter(ServerBehaviourDelegate.class, monitor);
+		
+		final boolean[] notified = new boolean[1];
+		
 		monitor = ProgressUtil.getMonitorFor(monitor);
 		
 		// add listener to the server
@@ -1423,10 +1552,11 @@ public class Server extends Base implements IServer {
 					int state = server.getServerState();
 					if (state == IServer.STATE_STARTED || state == IServer.STATE_STOPPED) {
 						// notify waiter
-						synchronized (mutex) {
+						synchronized (notified) {
 							try {
 								Trace.trace(Trace.FINEST, "synchronousStart notify");
-								mutex.notifyAll();
+								notified[0] = true;
+								notified.notifyAll();
 							} catch (Exception e) {
 								Trace.trace(Trace.SEVERE, "Error notifying server start", e);
 							}
@@ -1449,28 +1579,34 @@ public class Server extends Base implements IServer {
 			public void run() {
 				try {
 					int totalTimeout = serverTimeout;
+					if (totalTimeout < 0)
+						totalTimeout = 1;
 					boolean userCancelled = false;
 					int retryPeriod = 2500;
-					while (totalTimeout > 0 && !userCancelled && !timer.alreadyDone) {
+					while (!notified[0] && totalTimeout > 0 && !userCancelled && !timer.alreadyDone) {
 						Thread.sleep(retryPeriod);
-						totalTimeout -= retryPeriod;
-						if (monitor2.isCanceled()) {
+						if (serverTimeout > 0)
+							totalTimeout -= retryPeriod;
+						if (!notified[0] && !timer.alreadyDone && monitor2.isCanceled()) {
 							// user cancelled - set the server state to stopped
 							userCancelled = true;
 							setServerState(IServer.STATE_STOPPED);
 							// notify waiter
-							synchronized (mutex) {
+							synchronized (notified) {
 								Trace.trace(Trace.FINEST, "synchronousStart user cancelled.");
-								mutex.notifyAll();
+								notified[0] = true;
+								notified.notifyAll();
 							}
 						}
 					}
-					if (!userCancelled && !timer.alreadyDone) {
-						timer.timeout = true;
+					if (!userCancelled && !timer.alreadyDone && !notified[0]) {
 						// notify waiter
-						synchronized (mutex) {
+						synchronized (notified) {
 							Trace.trace(Trace.FINEST, "synchronousStart notify timeout");
-							mutex.notifyAll();
+							if (!timer.alreadyDone && totalTimeout <= 0)
+								timer.timeout = true;
+							notified[0] = true;
+							notified.notifyAll();
 						}
 					}
 				} catch (Exception e) {
@@ -1491,28 +1627,36 @@ public class Server extends Base implements IServer {
 			timer.alreadyDone = true;
 			throw e;
 		}
-	
+		if (monitor.isCanceled()) {
+			removeServerListener(listener);
+			timer.alreadyDone = true;
+			return;
+		}
+		
 		Trace.trace(Trace.FINEST, "synchronousStart 3");
-	
+		
 		// wait for it! wait for it! ...
-		synchronized (mutex) {
+		synchronized (notified) {
 			try {
-				while (!monitor.isCanceled() && !timer.timeout && !(getServerState() == IServer.STATE_STARTED || getServerState() == IServer.STATE_STOPPED))
-					mutex.wait();
+				while (!notified[0] && !monitor.isCanceled() && !timer.timeout
+						&& !(getServerState() == IServer.STATE_STARTED || getServerState() == IServer.STATE_STOPPED)) {
+					notified.wait();
+				}
 			} catch (Exception e) {
 				Trace.trace(Trace.SEVERE, "Error waiting for server start", e);
 			}
+			timer.alreadyDone = true;
 		}
 		removeServerListener(listener);
-		timer.alreadyDone = true;
 		
-		if (timer.timeout)
-			throw new CoreException(new Status(IStatus.ERROR, ServerPlugin.PLUGIN_ID, 0, NLS.bind(Messages.errorStartTimeout, new String[] { getName(), serverTimeout + "" }), null));
-		timer.alreadyDone = true;
+		if (timer.timeout) {
+			stop(false);
+			throw new CoreException(new Status(IStatus.ERROR, ServerPlugin.PLUGIN_ID, 0, NLS.bind(Messages.errorStartTimeout, new String[] { getName(), (serverTimeout / 1000) + "" }), null));
+		}
 		
 		if (getServerState() == IServer.STATE_STOPPED)
 			throw new CoreException(new Status(IStatus.ERROR, ServerPlugin.PLUGIN_ID, 0, NLS.bind(Messages.errorStartFailed, getName()), null));
-	
+		
 		Trace.trace(Trace.FINEST, "synchronousStart 4");
 	}
 
@@ -1649,25 +1793,28 @@ public class Server extends Base implements IServer {
 		}
 		final Timer timer = new Timer();
 		
-		Thread thread = new Thread() {
-			public void run() {
-				try {
-					Thread.sleep(120000);
-					if (!timer.alreadyDone) {
-						timer.timeout = true;
-						// notify waiter
-						synchronized (mutex) {
-							Trace.trace(Trace.FINEST, "stop notify timeout");
-							mutex.notifyAll();
+		final int serverTimeout = ((ServerType) getServerType()).getStopTimeout();
+		if (serverTimeout > 0) {
+			Thread thread = new Thread("Server stop timeout") {
+				public void run() {
+					try {
+						Thread.sleep(serverTimeout);
+						if (!timer.alreadyDone) {
+							timer.timeout = true;
+							// notify waiter
+							synchronized (mutex) {
+								Trace.trace(Trace.FINEST, "stop notify timeout");
+								mutex.notifyAll();
+							}
 						}
+					} catch (Exception e) {
+						Trace.trace(Trace.SEVERE, "Error notifying server stop timeout", e);
 					}
-				} catch (Exception e) {
-					Trace.trace(Trace.SEVERE, "Error notifying server stop timeout", e);
 				}
-			}
-		};
-		thread.setDaemon(true);
-		thread.start();
+			};
+			thread.setDaemon(true);
+			thread.start();
+		}
 	
 		// stop the server
 		stop(force);
@@ -1732,25 +1879,28 @@ public class Server extends Base implements IServer {
 		}
 		final Timer timer = new Timer();
 		
-		Thread thread = new Thread() {
-			public void run() {
-				try {
-					Thread.sleep(120000);
-					if (!timer.alreadyDone) {
-						timer.timeout = true;
-						// notify waiter
-						synchronized (mutex) {
-							Trace.trace(Trace.FINEST, "stop notify timeout");
-							mutex.notifyAll();
+		final int serverTimeout = ((ServerType) getServerType()).getStopTimeout();
+		if (serverTimeout > 0) {
+			Thread thread = new Thread("Synchronous server stop") {
+				public void run() {
+					try {
+						Thread.sleep(serverTimeout);
+						if (!timer.alreadyDone) {
+							timer.timeout = true;
+							// notify waiter
+							synchronized (mutex) {
+								Trace.trace(Trace.FINEST, "stop notify timeout");
+								mutex.notifyAll();
+							}
 						}
+					} catch (Exception e) {
+						Trace.trace(Trace.SEVERE, "Error notifying server stop timeout", e);
 					}
-				} catch (Exception e) {
-					Trace.trace(Trace.SEVERE, "Error notifying server stop timeout", e);
 				}
-			}
-		};
-		thread.setDaemon(true);
-		thread.start();
+			};
+			thread.setDaemon(true);
+			thread.start();
+		}
 	
 		// stop the server
 		stop(force);
@@ -1912,9 +2062,6 @@ public class Server extends Base implements IServer {
 		serverType = wc.serverType;
 		modules = wc.modules;
 		
-		int timestamp = wc.getTimestamp();
-		map.put("timestamp", Integer.toString(timestamp+1));
-		
 		// can never modify the following properties via the working copy
 		//serverState = wc.serverState;
 		delegate = wc.delegate;
@@ -2058,7 +2205,10 @@ public class Server extends Base implements IServer {
 		if (module == null)
 			throw new IllegalArgumentException("Module cannot be null");
 		try {
-			IModule[] children = getDelegate(monitor).getChildModules(module);
+			ServerDelegate sd = getDelegate(monitor);
+			if (sd == null)
+				return null;
+			IModule[] children = sd.getChildModules(module);
 			if (children != null && children.length == 1 && children[0].equals(module[module.length - 1]))
 				return null;
 			return children;
@@ -2097,7 +2247,10 @@ public class Server extends Base implements IServer {
 		if (module == null)
 			throw new IllegalArgumentException("Module cannot be null");
 		try {
-			boolean b = getBehaviourDelegate(monitor).canControlModule(module);
+			ServerBehaviourDelegate bd = getBehaviourDelegate(monitor);
+			if (bd == null)
+				return new Status(IStatus.ERROR, ServerPlugin.PLUGIN_ID, 0, Messages.errorRestartModule, null);
+			boolean b = bd.canControlModule(module);
 			if (b)
 				return Status.OK_STATUS;
 		} catch (Exception e) {
@@ -2139,7 +2292,7 @@ public class Server extends Base implements IServer {
 			Trace.trace(Trace.SEVERE, "Error calling delegate restartModule() " + toString(), e);
 		}
 	}
-	
+
 	/*
 	 * @see IServer#stopModule(IModule[], IOperationListener)
 	 */
@@ -2152,7 +2305,7 @@ public class Server extends Base implements IServer {
 			Trace.trace(Trace.SEVERE, "Error calling delegate restartModule() " + toString(), e);
 		}
 	}
-	
+
 	/*
 	 * @see IServer#restartModule(IModule[], IOperationListener, IProgressMonitor)
 	 */
@@ -2166,10 +2319,10 @@ public class Server extends Base implements IServer {
 			Trace.trace(Trace.SEVERE, "Error calling delegate restartModule() " + toString(), e);
 		}
 	}
-	
+
 	/**
 	 * Returns an array of IServerPorts that this server has.
-	 *
+	 * 
 	 * @param monitor
 	 * @return a possibly empty array of servers ports
 	 */
@@ -2181,7 +2334,7 @@ public class Server extends Base implements IServer {
 			return null;
 		}
 	}
-	
+
 	/**
 	 * Visit all the modules in the server with the given module visitor.
 	 * 
@@ -2252,7 +2405,7 @@ public class Server extends Base implements IServer {
 		moduleStatus.put(getKey(module), status);
 		//fireServerModuleStateChangeEvent(module);
 	}
-	
+
 	public IStatus getModuleStatus(IModule[] module) {
 		if (module == null)
 			throw new IllegalArgumentException("Module cannot be null");
